"""Data types and operations from Flight Planning Automated Testing Interface 0.1.0 OpenAPI"""

# This file is autogenerated; do not modify manually!

from __future__ import annotations

from enum import Enum
from typing import Dict, List, Optional

from uas_standards import Operation

from implicitdict import ImplicitDict, StringBasedDateTime


API_VERSION = "0.1.0"
"""Version of Flight Planning Automated Testing Interface OpenAPI specification from which the objects in this package were generated."""

class StatusResponseStatus(str, Enum):
    """The status of the USS automated testing interface.
    - `Starting`: the USS is starting and the automated test driver should wait before sending requests.
    - `Ready`: the USS is ready to receive test requests.
    """

    Starting = "Starting"
    Ready = "Ready"


class StatusResponse(ImplicitDict):
    status: StatusResponseStatus
    """The status of the USS automated testing interface.
    - `Starting`: the USS is starting and the automated test driver should wait before sending requests.
    - `Ready`: the USS is ready to receive test requests.
    """

    system_version: Optional[str]
    """Arbitrary string representing the version of the USS system to be tested."""

    api_name: Optional[str]
    """Indication of the API implemented at this URL.  Must be "Flight Planning Automated Testing Interface"."""

    api_version: Optional[str]
    """Indication of the API version implemented at this URL.  Must be "v0.1.0" when implementing this version of the API."""


class FlightPlanAdditionalInformation(ImplicitDict):
    """Any information relevant to a particular jurisdiction or use case not described in the standard schema. The keys and values must be agreed upon between the test designers and USSs under test."""



class BasicFlightPlanInformationUsageState(str, Enum):
    """User's current usage of the flight plan.
    `Planned`: The user intends to fly according to this flight plan, but is not currently using the defined
      area with an active UAS.

    `InUse`: The user is currently using the defined area with an active UAS.
    """

    Planned = "Planned"
    InUse = "InUse"


class BasicFlightPlanInformationUasState(str, Enum):
    """State of the user's UAS associated with this flight plan.

      - `Nominal`: The user or UAS reports or implies that it is performing nominally.

      - `OffNominal`: The user or UAS reports or implies that it is temporarily not performing nominally, but
        expects to be able to recover to normal operation.

      - `Contingent`: The user or UAS reports or implies that it is not performing nominally and may be unable
        to recover to normal operation.
    """

    Nominal = "Nominal"
    OffNominal = "OffNominal"
    Contingent = "Contingent"


class UpsertFlightPlanResponseResult(str, Enum):
    """The result of the flight plan submission. If any option other than `Planned` or `ReadyToFly` is specified, the `notes` field should be populated with the reason for the unsuccessful outcome.

      - `Planned`: The data submitted in the flight plan intent was valid and the flight plan was successfully processed by the USS and is now authorized, but the user may not yet start flying (even if within the time bounds of the flight plan).

      - `ReadyToFly`: The flight plan is ready for the operator to begin flying within the bounds (including time) of the flight plan.

      - `Rejected`: The data provided in the flight plan was invalid and/or could not be used to successfully authorize the flight.  The reason for rejection may include a disallowed conflict with another flight.

      - `Failed`: The USS was not able to successfully authorize the flight plan due to a problem with the USS or a downstream system

      - `NotSupported`: The USS does not support the attempted interaction.  For instance, if the request specified a high-priority flight and the USS does not support management of high-priority flights.
    """

    Planned = "Planned"
    ReadyToFly = "ReadyToFly"
    Rejected = "Rejected"
    Failed = "Failed"
    NotSupported = "NotSupported"


UpsertFlightPlanResponseIncludesAdvisories = str
"""Indication of whether any advisories or conditions were provided to the user along with the result of this flight planning attempt.

  - `Unknown`: It is unknown or irrelevant whether advisories or conditions were provided to the user

  - `Yes`: At least one advisory or condition was provided to the user.

  - `No`: No advisories or conditions were provided to the user.

Acceptable values:
* Unknown
* True
* False
"""


class UpsertFlightPlanResponse(ImplicitDict):
    result: UpsertFlightPlanResponseResult
    """The result of the flight plan submission. If any option other than `Planned` or `ReadyToFly` is specified, the `notes` field should be populated with the reason for the unsuccessful outcome.

      - `Planned`: The data submitted in the flight plan intent was valid and the flight plan was successfully processed by the USS and is now authorized, but the user may not yet start flying (even if within the time bounds of the flight plan).

      - `ReadyToFly`: The flight plan is ready for the operator to begin flying within the bounds (including time) of the flight plan.

      - `Rejected`: The data provided in the flight plan was invalid and/or could not be used to successfully authorize the flight.  The reason for rejection may include a disallowed conflict with another flight.

      - `Failed`: The USS was not able to successfully authorize the flight plan due to a problem with the USS or a downstream system

      - `NotSupported`: The USS does not support the attempted interaction.  For instance, if the request specified a high-priority flight and the USS does not support management of high-priority flights.
    """

    notes: Optional[str]
    """Human-readable explanation of the observed result.  This explanation may be made available to a human reviewing the test results, and ideally should explain why an undesirable result was obtained.  For instance, if the injection attempt Failed, then these notes may indicate that the attempt failed because the DSS indicated 400 to a valid request (perhaps also including the valid request as proof)."""

    includes_advisories: Optional[UpsertFlightPlanResponseIncludesAdvisories] = "Unknown"
    """Indication of whether any advisories or conditions were provided to the user along with the result of this flight planning attempt.

      - `Unknown`: It is unknown or irrelevant whether advisories or conditions were provided to the user

      - `Yes`: At least one advisory or condition was provided to the user.

      - `No`: No advisories or conditions were provided to the user.
    """


class DeleteFlightPlanResponseResult(str, Enum):
    """The result of attempted flight plan cancellation/closure.

      - `Closed`: The flight plan was closed successfully by the USS and is now out of the UTM system.

      - `Failed`: The flight plan could not be closed successfully by the USS.
    """

    Closed = "Closed"
    Failed = "Failed"


class DeleteFlightPlanResponse(ImplicitDict):
    result: DeleteFlightPlanResponseResult
    """The result of attempted flight plan cancellation/closure.

      - `Closed`: The flight plan was closed successfully by the USS and is now out of the UTM system.

      - `Failed`: The flight plan could not be closed successfully by the USS.
    """

    notes: Optional[str]
    """Human-readable explanation of the observed result."""


class ClearAreaOutcome(ImplicitDict):
    success: Optional[bool] = False
    """True if, and only if, all flight plans in the specified area managed by the USS were canceled and removed."""

    message: Optional[str]
    """If the USS was unable to clear the entire area, this message can provide information on the problem encountered."""


class ClearAreaResponse(ImplicitDict):
    outcome: ClearAreaOutcome


Priority = int
"""Ordinal priority of the operational intent, as defined in ASTM F3548-21."""


class FlightAuthorisationDataOperationCategory(str, Enum):
    """Category of UAS operation (‘open’, ‘specific’, ‘certified’) as defined in COMMISSION DELEGATED REGULATION (EU) 2019/945. Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 4."""

    Unknown = "Unknown"
    Open = "Open"
    Specific = "Specific"
    Certified = "Certified"


class OperationMode(str, Enum):
    """Specify if the operation is a `VLOS` or `BVLOS` operation. Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 2."""

    Undeclared = "Undeclared"
    Vlos = "Vlos"
    Bvlos = "Bvlos"


class UASClass(str, Enum):
    """Specify the class of the UAS to be flown, the specifition matches EASA class identification label categories. UAS aircraft class as defined in COMMISSION DELEGATED REGULATION (EU) 2019/945 (C0 to C4) and COMMISSION DELEGATED REGULATION (EU) 2020/1058 (C5 and C6). This field is required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 4."""

    Other = "Other"
    C0 = "C0"
    C1 = "C1"
    C2 = "C2"
    C3 = "C3"
    C4 = "C4"
    C5 = "C5"
    C6 = "C6"


class RPAS26FlightDetailsOperatorType(str, Enum):
    """The type of operator."""

    Recreational = "Recreational"
    CommercialExcluded = "CommercialExcluded"
    ReOC = "ReOC"


class RPAS26FlightDetailsAircraftType(str, Enum):
    """Type of vehicle being used as per ASTM F3411-22a."""

    NotDeclared = "NotDeclared"
    Aeroplane = "Aeroplane"
    Helicopter = "Helicopter"
    Gyroplane = "Gyroplane"
    HybridLift = "HybridLift"
    Ornithopter = "Ornithopter"
    Glider = "Glider"
    Kite = "Kite"
    FreeBalloon = "FreeBalloon"
    CaptiveBalloon = "CaptiveBalloon"
    Airship = "Airship"
    FreeFallOrParachute = "FreeFallOrParachute"
    Rocket = "Rocket"
    TetheredPoweredAircraft = "TetheredPoweredAircraft"
    GroundObstacle = "GroundObstacle"
    Other = "Other"


class RPAS26FlightDetailsFlightProfile(str, Enum):
    """Type of flight profile."""

    AutomatedGrid = "AutomatedGrid"
    AutomatedWaypoint = "AutomatedWaypoint"
    Manual = "Manual"


class RPAS26FlightDetails(ImplicitDict):
    """Information about a flight necessary to plan successfully using the RPAS Platform Operating Rules version 2.6."""

    operator_type: Optional[RPAS26FlightDetailsOperatorType]
    """The type of operator."""

    uas_serial_numbers: Optional[List[str]]
    """The list of UAS/drone serial numbers that will be operated during the operation."""

    uas_registration_numbers: Optional[List[str]]
    """The list of UAS/drone registration numbers that will be operated during the operation."""

    aircraft_type: Optional[RPAS26FlightDetailsAircraftType]
    """Type of vehicle being used as per ASTM F3411-22a."""

    flight_profile: Optional[RPAS26FlightDetailsFlightProfile]
    """Type of flight profile."""

    pilot_license_number: Optional[str]
    """License number for the pilot."""

    pilot_phone_number: Optional[str]
    """Contact phone number for the pilot."""

    operator_number: Optional[str]
    """Operator number."""


Longitude = float
"""Degrees of longitude east of the Prime Meridian, with reference to the WGS84 ellipsoid."""


Latitude = float
"""Degrees of latitude north of the equator, with reference to the WGS84 ellipsoid."""


class RadiusUnits(str, Enum):
    """FIXM-compatible units.  Only meters ("M") are acceptable for UTM."""

    M = "M"


class Radius(ImplicitDict):
    value: float
    """Distance from the centerpoint of a circular area, along the WGS84 ellipsoid."""

    units: RadiusUnits = RadiusUnits.M
    """FIXM-compatible units.  Only meters ("M") are acceptable for UTM."""


class AltitudeReference(str, Enum):
    """A code indicating the reference for a vertical distance. See AIXM 5.1 and FIXM 4.2.0."""

    W84 = "W84"
    SFC = "SFC"


class AltitudeUnits(str, Enum):
    """The reference quantities used to express the value of altitude. See FIXM 4.2. Currently, UTM only allows meters with no immediate plans to allow other options."""

    M = "M"


class Altitude(ImplicitDict):
    value: float
    """The numeric value of the altitude. Note that min and max values are added as a sanity check. As use cases evolve and more options are made available in terms of units of measure or reference systems, these bounds may be re-evaluated."""

    reference: AltitudeReference
    """A code indicating the reference for a vertical distance. See AIXM 5.1 and FIXM 4.2.0."""

    units: AltitudeUnits = AltitudeUnits.M
    """The reference quantities used to express the value of altitude. See FIXM 4.2. Currently, UTM only allows meters with no immediate plans to allow other options."""


class TimeFormat(str, Enum):
    RFC3339 = "RFC3339"


class Time(ImplicitDict):
    value: StringBasedDateTime
    """RFC3339-formatted time/date string.  The time zone must be 'Z'."""

    format: TimeFormat = TimeFormat.RFC3339


UUIDv4Format = str
"""String whose format matches a version-4 UUID according to RFC 4122."""


class ASTMF354821OpIntentInformation(ImplicitDict):
    """Information provided about a flight plan that is necessary for ASTM F3548-21."""

    priority: Optional[Priority]


class FlightAuthorisationData(ImplicitDict):
    """The details of a UAS flight authorization request, as received from the user.

    Note that a full description of a flight authorisation must include mandatory information required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664 for an UAS flight authorisation request. Reference: https://eur-lex.europa.eu/legal-content/EN/TXT/HTML/?uri=CELEX:32021R0664&from=EN#d1e32-178-1
    """

    uas_serial_number: str
    """Unique serial number of the unmanned aircraft or, if the unmanned aircraft is privately built, the unique serial number of the add-on. This is expressed in the ANSI/CTA-2063 Physical Serial Number format. Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 1."""

    operation_mode: OperationMode

    operation_category: FlightAuthorisationDataOperationCategory
    """Category of UAS operation (‘open’, ‘specific’, ‘certified’) as defined in COMMISSION DELEGATED REGULATION (EU) 2019/945. Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 4."""

    uas_class: UASClass

    identification_technologies: List[str]
    """Technology used to identify the UAS. Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 6."""

    uas_type_certificate: Optional[str]
    """Provisional field. Not applicable as of September 2021. Required only if `uas_class` is set to `other` by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 4."""

    connectivity_methods: List[str]
    """Connectivity methods. Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 7."""

    endurance_minutes: int
    """Endurance of the UAS. This is expressed in minutes. Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 8."""

    emergency_procedure_url: str
    """The URL at which the applicable emergency procedure in case of a loss of command and control link may be retrieved. Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 9."""

    operator_id: str
    """Registration number of the UAS operator.
    The format is defined in EASA Easy Access Rules for Unmanned Aircraft Systems GM1 to AMC1
    Article 14(6) Registration of UAS operators and ‘certified’ UAS.
    Required by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 10.
    """

    uas_id: Optional[str]
    """When applicable, the registration number of the unmanned aircraft.
    This is expressed using the nationality and registration mark of the unmanned aircraft in
    line with ICAO Annex 7.
    Specified by ANNEX IV of COMMISSION IMPLEMENTING REGULATION (EU) 2021/664, paragraph 10.
    """


class LatLngPoint(ImplicitDict):
    """Point on the earth's surface."""

    lng: Longitude

    lat: Latitude


class Polygon(ImplicitDict):
    """An enclosed area on the earth. The bounding edges of this polygon are defined to be the shortest paths between connected vertices.  This means, for instance, that the edge between two points both defined at a particular latitude is not generally contained at that latitude. The winding order must be interpreted as the order which produces the smaller area. The path between two vertices is defined to be the shortest possible path between those vertices. Edges may not cross. Vertices may not be duplicated.  In particular, the final polygon vertex must not be identical to the first vertex."""

    vertices: List[LatLngPoint]


class Circle(ImplicitDict):
    """A circular area on the surface of the earth."""

    center: Optional[LatLngPoint]

    radius: Optional[Radius]


class Volume3D(ImplicitDict):
    """A three-dimensional geographic volume consisting of a vertically-extruded shape. Exactly one outline must be specified."""

    outline_circle: Optional[Circle]
    """A circular geographic shape on the surface of the earth."""

    outline_polygon: Optional[Polygon]
    """A polygonal geographic shape on the surface of the earth."""

    altitude_lower: Optional[Altitude]
    """Minimum bounding altitude of this volume. Must be less than altitude_upper, if specified."""

    altitude_upper: Optional[Altitude]
    """Maximum bounding altitude of this volume. Must be greater than altitude_lower, if specified."""


class Volume4D(ImplicitDict):
    """Contiguous block of geographic spacetime."""

    volume: Volume3D

    time_start: Optional[Time]
    """Beginning time of this volume. Must be before time_end."""

    time_end: Optional[Time]
    """End time of this volume. Must be after time_start."""


class BasicFlightPlanInformation(ImplicitDict):
    """Basic information about a flight plan that an operator and/or UAS can be expected to provide in most flight planning scenarios."""

    usage_state: BasicFlightPlanInformationUsageState
    """User's current usage of the flight plan.
    `Planned`: The user intends to fly according to this flight plan, but is not currently using the defined
      area with an active UAS.

    `InUse`: The user is currently using the defined area with an active UAS.
    """

    uas_state: BasicFlightPlanInformationUasState
    """State of the user's UAS associated with this flight plan.

      - `Nominal`: The user or UAS reports or implies that it is performing nominally.

      - `OffNominal`: The user or UAS reports or implies that it is temporarily not performing nominally, but
        expects to be able to recover to normal operation.

      - `Contingent`: The user or UAS reports or implies that it is not performing nominally and may be unable
        to recover to normal operation.
    """

    nominal_area: Optional[List[Volume4D]] = []
    """User nominally intends or intended to fly in this entire area.  If authorizations are required and cannot be granted to cover this entire area, the plan should be rejected."""

    off_nominal_area: Optional[List[Volume4D]] = []
    """While experiencing an off-nominal situation, the user's aircraft may travel anywhere in this area.  If the flight_state is nominal, this field must be empty or omitted."""


class ClearAreaRequest(ImplicitDict):
    request_id: str
    """Unique string identifying this request.  If a second request with an identical ID is received, the USS may return the same response from the previous operation rather than attempting to clear the area again (the USS may also attempt to clear the area again)."""

    extent: Volume4D
    """The USS should cancel and remove any flight plan where any part of that flight plan intersects this area."""


class FlightPlan(ImplicitDict):
    """Details of user's intent to create or modify a flight plan."""

    basic_information: BasicFlightPlanInformation

    astm_f3548_21: Optional[ASTMF354821OpIntentInformation]

    uspace_flight_authorisation: Optional[FlightAuthorisationData]

    rpas_operating_rules_2_6: Optional[RPAS26FlightDetails]

    additional_information: Optional[FlightPlanAdditionalInformation]
    """Any information relevant to a particular jurisdiction or use case not described in the standard schema. The keys and values must be agreed upon between the test designers and USSs under test."""


class UpsertFlightPlanRequest(ImplicitDict):
    intended_flight: FlightPlan

    request_id: UUIDv4Format
    """ID uniquely identifying the upsertion request.  If additional requests are received with the same request_id, the response from the first request should be returned, or an error indicated."""


class OperationID(str, Enum):
    GetStatus = "GetStatus"
    ClearArea = "ClearArea"
    UpsertFlightPlan = "UpsertFlightPlan"
    DeleteFlightPlan = "DeleteFlightPlan"


OPERATIONS: Dict[OperationID, Operation] = {
    OperationID.GetStatus: Operation(
        id="GetStatus",
        path="/status",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: StatusResponse,
            401: None,
            403: None,
            404: None,
        }
    ),
    OperationID.ClearArea: Operation(
        id="ClearArea",
        path="/clear_area_requests",
        verb="POST",
        request_body_type=ClearAreaRequest,
        response_body_type={
            200: ClearAreaResponse,
            401: None,
            403: None,
        }
    ),
    OperationID.UpsertFlightPlan: Operation(
        id="UpsertFlightPlan",
        path="/flight_plans/{flight_plan_id}",
        verb="PUT",
        request_body_type=UpsertFlightPlanRequest,
        response_body_type={
            200: UpsertFlightPlanResponse,
            401: None,
            403: None,
            409: None,
        }
    ),
    OperationID.DeleteFlightPlan: Operation(
        id="DeleteFlightPlan",
        path="/flight_plans/{flight_plan_id}",
        verb="DELETE",
        request_body_type=None,
        response_body_type={
            200: DeleteFlightPlanResponse,
            401: None,
            403: None,
            404: None,
        }
    ),
}



"""Data types and operations from Remote ID Display Data Observation 0.0.2 OpenAPI"""

# This file is autogenerated; do not modify manually!

from __future__ import annotations

from enum import Enum
from typing import Dict, List, Optional

from uas_standards import Operation

from implicitdict import ImplicitDict


API_VERSION = "0.0.2"
"""Version of Remote ID Display Data Observation OpenAPI specification from which the objects in this package were generated."""

class UAS(ImplicitDict):
    """UAS information"""

    id: Optional[str]
    """Observed arbitrary and format agnostic (string) identification of a UAS."""


class RIDOperationalStatus(str, Enum):
    """Indicates operational status of associated aircraft."""

    Undeclared = "Undeclared"
    Ground = "Ground"
    Airborne = "Airborne"
    Emergency = "Emergency"
    RemoteIDSystemFailure = "RemoteIDSystemFailure"


RIDTrack = float
"""Direction of flight expressed as a "True North-based" ground track angle.  This value is provided in degrees East of North with a minimum resolution of 1 degree."""


RIDSpeed = float
"""Ground speed of flight in meters per second."""


Longitude = float
"""Degrees of longitude east of the Prime Meridian, with reference to the WGS84 ellipsoid."""


Latitude = float
"""Degrees of latitude north of the equator, with reference to the WGS84 ellipsoid."""


class OperatorAltitudeAltitudeType(str, Enum):
    """Source of data for the altitude field."""

    Takeoff = "Takeoff"
    Dynamic = "Dynamic"
    Fixed = "Fixed"


Altitude = float
"""An altitude, in meters, above the WGS84 ellipsoid."""


class RIDHeightReference(str, Enum):
    """The reference datum above which the height is reported."""

    TakeoffLocation = "TakeoffLocation"
    GroundLevel = "GroundLevel"


class RIDHeight(ImplicitDict):
    """A relative altitude for the purposes of remote ID."""

    distance: float
    """Distance above reference datum.  This value is provided in meters and must have a minimum resolution of 1 meter."""

    reference: RIDHeightReference
    """The reference datum above which the height is reported."""


class CurrentState(ImplicitDict):
    """Current state of an aircraft."""

    timestamp: Optional[str]
    """Time at which the message containing the current state information was generated by the display provider."""

    operational_status: Optional[RIDOperationalStatus]

    track: Optional[RIDTrack]

    speed: Optional[RIDSpeed]


class Position(ImplicitDict):
    """A position on Earth."""

    lat: Latitude

    lng: Longitude

    alt: Optional[Altitude]

    height: Optional[RIDHeight]


class Path(ImplicitDict):
    """Path followed by a flight."""

    positions: List[Position]
    """Sequential positions available for a flight."""


class LatLngPoint(ImplicitDict):
    """Point on the earth's surface."""

    lng: Longitude

    lat: Latitude


class OperatorAltitude(ImplicitDict):
    """Altitude associated with the Remote Pilot"""

    altitude: Optional[Altitude]

    altitude_type: Optional[OperatorAltitudeAltitudeType]
    """Source of data for the altitude field."""


class Flight(ImplicitDict):
    id: str
    """Identifier of flight that may be used to obtain details about the flight. This is not necessarily the UTM/flight ID in the remote ID system."""

    current_state: Optional[CurrentState]

    most_recent_position: Optional[Position]
    """Most recent position known for the flight."""

    recent_paths: Optional[List[Path]]
    """Paths the flight recently traveled, if available."""


class Operator(ImplicitDict):
    """Operator's information"""

    id: Optional[str]
    """CAA-issued registration/license ID for the remote pilot or operator."""

    location: Optional[LatLngPoint]

    altitude: Optional[OperatorAltitude]


class Cluster(ImplicitDict):
    """A general area containing one or more flight."""

    corners: List[Position]
    """Two opposite corners of a rectangular lat-lng box bounding the cluster."""

    area_sqm: float
    """Area of the cluster in square meters."""

    number_of_flights: int
    """Number of flights within the cluster."""


class GetDisplayDataResponse(ImplicitDict):
    """Response to a request for current data that would be visualized by a Display Application."""

    flights: Optional[List[Flight]] = []
    """Current information for set of discovered flights whose precise locations are known."""

    clusters: Optional[List[Cluster]] = []
    """Current information for sets of discovered flights whose precise locations are not known."""


class GetDetailsResponse(ImplicitDict):
    """Response to a request to get details about a flight."""

    operator: Optional[Operator]

    uas: Optional[UAS]


class OperationID(str, Enum):
    GetDisplayData = "getDisplayData"
    GetDetails = "getDetails"


OPERATIONS: Dict[OperationID, Operation] = {
    OperationID.GetDisplayData: Operation(
        id="getDisplayData",
        path="/display_data",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetDisplayDataResponse,
        }
    ),
    OperationID.GetDetails: Operation(
        id="getDetails",
        path="/display_data/{id}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetDetailsResponse,
            404: None,
        }
    ),
}



"""Data types and operations from UTM API (USS->DSS and USS->USS) 1.0.0 OpenAPI"""

# This file is autogenerated; do not modify manually!

from __future__ import annotations

from enum import Enum
from typing import Dict, List, Optional

from uas_standards import Operation

from implicitdict import ImplicitDict, StringBasedDateTime


API_VERSION = "1.0.0"
"""Version of UTM API (USS->DSS and USS->USS) OpenAPI specification from which the objects in this package were generated."""

UUIDv4Format = str
"""String whose format matches a version-4 UUID according to RFC 4122."""


EntityID = UUIDv4Format


EntityOVN = str
"""A token associated with a particular UTM Entity+version created by the DSS upon creation or modification of an Entity reference and provided to the client creating or modifying the Entity reference.  The EntityOVN is stored privately by the DSS and then compared against entries in a Key provided to mutate the airspace.  The EntityOVN is also provided by the client whenever that client transmits the full information of the Entity (either via GET, or via a subscription notification)."""


SubscriptionID = UUIDv4Format


Key = List[EntityOVN]


class TimeFormat(str, Enum):
    RFC3339 = "RFC3339"


class Time(ImplicitDict):
    value: StringBasedDateTime
    """RFC3339-formatted time/date string.  The time zone must be 'Z'."""

    format: TimeFormat = TimeFormat.RFC3339


class RadiusUnits(str, Enum):
    """FIXM-compatible units.  Only meters ("M") are acceptable for UTM."""

    M = "M"


class Radius(ImplicitDict):
    value: float
    """Distance from the centerpoint of a circular area, along the WGS84 ellipsoid."""

    units: RadiusUnits = RadiusUnits.M
    """FIXM-compatible units.  Only meters ("M") are acceptable for UTM."""


class AltitudeReference(str, Enum):
    """A code indicating the reference for a vertical distance. See AIXM 5.1 and FIXM 4.2.0. Currently, UTM only allows WGS84 with no immediate plans to allow other options. FIXM and AIXM allow for 'SFC' which is equivalent to AGL."""

    W84 = "W84"


class AltitudeUnits(str, Enum):
    """The reference quantities used to express the value of altitude. See FIXM 4.2. Currently, UTM only allows meters with no immediate plans to allow other options."""

    M = "M"


class Altitude(ImplicitDict):
    value: float
    """The numeric value of the altitude. Note that min and max values are added as a sanity check. As use cases evolve and more options are made available in terms of units of measure or reference systems, these bounds may be re-evaluated."""

    reference: AltitudeReference = AltitudeReference.W84
    """A code indicating the reference for a vertical distance. See AIXM 5.1 and FIXM 4.2.0. Currently, UTM only allows WGS84 with no immediate plans to allow other options. FIXM and AIXM allow for 'SFC' which is equivalent to AGL."""

    units: AltitudeUnits = AltitudeUnits.M
    """The reference quantities used to express the value of altitude. See FIXM 4.2. Currently, UTM only allows meters with no immediate plans to allow other options."""


Latitude = float
"""Degrees of latitude north of the equator, with reference to the WGS84 ellipsoid."""


Longitude = float
"""Degrees of longitude east of the Prime Meridian, with reference to the WGS84 ellipsoid."""


class LatLngPoint(ImplicitDict):
    """Point on the earth's surface."""

    lng: Longitude

    lat: Latitude


class Circle(ImplicitDict):
    """A circular area on the surface of the earth."""

    center: Optional[LatLngPoint]

    radius: Optional[Radius]


class ErrorResponse(ImplicitDict):
    """Human-readable string returned when an error occurs as a result of a USS - DSS transaction."""

    message: Optional[str]
    """Human-readable message indicating what error occurred and/or why."""


SubscriptionNotificationIndex = int
"""Tracks the notifications sent for a subscription so the subscriber can detect missed notifications more easily."""


UssBaseURL = str
"""The base URL of a USS implementation of part or all of the USS-USS API. Per the USS-USS API, the full URL of a particular resource can be constructed by appending, e.g., `/uss/v1/{resource}/{id}` to this base URL. Accordingly, this URL may not have a trailing '/' character."""


class OperationalIntentState(str, Enum):
    """State of an operational intent.
    'Accepted': Operational intent is created and shared, but not yet in use; see standard text for more details.
    The create or update request for this operational intent reference must include a Key containing all OVNs for all relevant Entities.
    'Activated': Operational intent is in active use; see standard text for more details.
    The create or update request for this operational intent reference must include a Key containing all OVNs for all relevant Entities.
    'Nonconforming': UA is temporarily outside its volumes, but the situation is expected to be recoverable; see standard text for more details.
    In this state, the `/uss/v1/operational_intents/{entityid}/telemetry` USS-USS endpoint should respond, if available, to queries from USS peers.  The create or update request for this operational intent may omit a Key in this case because the operational intent is being adjusted as flown and cannot necessarily deconflict.
    'Contingent': UA is considered unrecoverably unable to conform with its coordinate operational intent; see standard text for more details.
    This state must transition to Ended.  In this state, the `/uss/v1/operational_intents/{entityid}/telemetry` USS-USS endpoint should respond, if available, to queries from USS peers.  The create or update request for this operational intent may omit a Key in this case because the operational intent is being adjusted as flown and cannot necessarily deconflict.
    """

    Accepted = "Accepted"
    Activated = "Activated"
    Nonconforming = "Nonconforming"
    Contingent = "Contingent"


OperationalIntentUssBaseURL = UssBaseURL


ConstraintUssBaseURL = UssBaseURL


Priority = int
"""Ordinal priority of the operational intent, as defined by the regulator.  Operational intents with lesser values are lower priority than all operational intents with greater values.  A lower-priority operational intent may not create a conflict with a higher-priority operational intent.  A higher-priority operational intent may create a conflict with a lower-priority operational intent.  The regulator specifies whether an operational intent may create a conflict with other operational intents of the same priority."""


class PositionAccuracyVertical(str, Enum):
    """Vertical error that is likely to be present in this reported position. This is the GVA enumeration from ADS-B, plus some finer values for UAS."""

    VAUnknown = "VAUnknown"
    VA150mPlus = "VA150mPlus"
    VA150m = "VA150m"
    VA45m = "VA45m"
    VA25m = "VA25m"
    VA10m = "VA10m"
    VA3m = "VA3m"
    VA1m = "VA1m"


class PositionAccuracyHorizontal(str, Enum):
    """Horizontal error that is likely to be present in this reported position. This is the NACp enumeration from ADS-B, plus 1m for a more complete range for UAS."""

    HAUnknown = "HAUnknown"
    HA10NMPlus = "HA10NMPlus"
    HA10NM = "HA10NM"
    HA4NM = "HA4NM"
    HA2NM = "HA2NM"
    HA1NM = "HA1NM"
    HA05NM = "HA05NM"
    HA03NM = "HA03NM"
    HA01NM = "HA01NM"
    HA005NM = "HA005NM"
    HA30m = "HA30m"
    HA10m = "HA10m"
    HA3m = "HA3m"
    HA1m = "HA1m"


class Position(ImplicitDict):
    """Location of the vehicle (UAS) as reported for UTM. Note: 'accuracy' values are required when extrapolated field is true."""

    longitude: Optional[Longitude]

    latitude: Optional[Latitude]

    accuracy_h: Optional[PositionAccuracyHorizontal]

    accuracy_v: Optional[PositionAccuracyVertical]

    extrapolated: Optional[bool] = False
    """True if this position was generated primarily by computation rather than primarily from a direct instrument measurement."""

    altitude: Optional[Altitude]


class VelocityUnitsSpeed(str, Enum):
    MetersPerSecond = "MetersPerSecond"


class Velocity(ImplicitDict):
    speed: float
    """Ground speed in meters/second."""

    units_speed: VelocityUnitsSpeed = VelocityUnitsSpeed.MetersPerSecond

    track: Optional[float] = 0
    """Direction of flight expressed as a "True North-based" ground track angle. This value is provided in degrees East of North with a minimum resolution of 1 degree. A value of 360 indicates invalid, no value, or unknown."""


class UssAvailabilityState(str, Enum):
    """A USS is presumed to be in the Unknown state absent indication otherwise by a USS with availability arbitration scope.  Upon determination via availability arbitration, a USS is Down when it does not respond appropriately, and a Down USS may not perform the following operations in the DSS:
    * Create an operational intent in the Accepted or Activated states
    * Modify an operational intent whose new or unchanged state is Accepted or Activated
    * Delete an operational intent
    A USS in the Unknown state possesses all the capabilities, within the DSS, of a USS in the Normal state.
    """

    Unknown = "Unknown"
    Normal = "Normal"
    Down = "Down"


class SetUssAvailabilityStatusParameters(ImplicitDict):
    old_version: str = ""
    """Version of USS's availability to change, for consistent read-modify-write operations and consistent retry behavior."""

    availability: UssAvailabilityState


class ExchangeRecordRecorderRole(str, Enum):
    """A coded value that indicates the role of the logging USS: 'Client' (initiating a request to a remote USS) or 'Server' (handling a request from a remote USS)"""

    Client = "Client"
    Server = "Server"


class ExchangeRecord(ImplicitDict):
    """Details of a request/response data exchange."""

    url: str
    """Full URL of request."""

    method: str
    """HTTP verb used by requestor (e.g., "PUT," "GET," etc.)"""

    headers: Optional[List[str]] = []
    """Set of headers associated with request or response. Requires 'Authorization:' field (at a minimum)"""

    recorder_role: ExchangeRecordRecorderRole
    """A coded value that indicates the role of the logging USS: 'Client' (initiating a request to a remote USS) or 'Server' (handling a request from a remote USS)"""

    request_time: Time
    """The time at which the request was sent/received."""

    request_body: Optional[str] = ""
    """Base64-encoded body content sent/received as a request."""

    response_time: Optional[Time]
    """The time at which the response was sent/received."""

    response_body: Optional[str] = ""
    """Base64-encoded body content sent/received in response to request."""

    response_code: Optional[int] = 0
    """HTTP response code sent/received in response to request."""

    problem: Optional[str]
    """'Human-readable description of the problem with the exchange, if any.'"""


class ErrorReport(ImplicitDict):
    """A report informing a server of a communication problem."""

    report_id: Optional[str]
    """ID assigned by the server receiving the report.  Not populated when submitting a report."""

    exchange: ExchangeRecord
    """The request (by this USS) and response associated with the error."""


class PlanningRecord(ImplicitDict):
    """A record of a single attempt to (successfully or unsuccessfully) create or modify an operational intent."""

    time: Time
    """Time that this planning event occurred"""

    ovns: List[EntityOVN] = []
    """OVNs the planning USS was aware of when it was planning the operational intent"""

    missing_operational_intents: Optional[List[EntityID]] = []
    """List of missing operational intents (for planning attempts that were denied by the DSS with code 409)"""

    missing_constraints: Optional[List[EntityID]] = []
    """List of missing constraints (for planning attempts that were denied by the DSS with code 409)"""

    operational_intent_id: Optional[EntityID]
    """ID of the operational intent being planned"""

    problem: Optional[str]
    """A free text description of the problem(s) encountered during this planning attempt."""


class UserNotificationRecordNotificationTriggeringEvent(str, Enum):
    """Requirement ID that pertains to the given notification"""

    GEN0400 = "GEN0400"
    GEN0405 = "GEN0405"
    SCD0090 = "SCD0090"
    SCD0095 = "SCD0095"
    ACM0010 = "ACM0010"
    CMSA0115 = "CMSA0115"
    CMSA0300 = "CMSA0300"
    CSTP0005 = "CSTP0005"
    CSTP0010 = "CSTP0010"
    CSTP0020 = "CSTP0020"
    CSTP0025 = "CSTP0025"
    CSTP0030 = "CSTP0030"
    CSTO0035 = "CSTO0035"


class UserNotificationRecord(ImplicitDict):
    """User notification record."""

    triggering_event_time: Time
    """Time of the notification triggering event"""

    notification_time: Time
    """Time at which the user was notified"""

    notification_details: Optional[str]
    """Description of information that was provided to the user, as per the referenced notification_triggering_event requirement"""

    notification_triggering_event: UserNotificationRecordNotificationTriggeringEvent
    """Requirement ID that pertains to the given notification"""


class UserInputRecordInputTriggeringEvent(str, Enum):
    """Requirement ID that pertains to the given notification"""

    OPIN0040 = "OPIN0040"
    CMSA0010 = "CMSA0010"
    CMSA0025 = "CMSA0025"
    CMSA0100 = "CMSA0100"
    CMSA0105 = "CMSA0105"
    CMSA0110 = "CMSA0110"
    CMSA0200 = "CMSA0200"
    CMSA0205 = "CMSA0205"
    CMSA0210 = "CMSA0210"
    CMSA0215 = "CMSA0215"


class UserInputRecord(ImplicitDict):
    """User input record"""

    triggering_event_time: Time
    """Time in which user input was received by the USS"""

    operational_intent_id: EntityID
    """ID of the operational_intent ID pertaining to the user input"""

    input_triggering_event: UserInputRecordInputTriggeringEvent
    """Requirement ID that pertains to the given notification"""

    input_details: Optional[str]
    """Description of the information that was provided by the user, as per the referenced input_triggering_event requirement"""


class OperatorAssociation(ImplicitDict):
    """Association between an operational intent and the operator of that operational intent"""

    operational_intent_id: EntityID
    """ID of operational intent to which this association pertains"""

    operator_id: str
    """Unique identifier of the operator responsible for the operational intent"""


class ConstraintProviderAssociation(ImplicitDict):
    """Association between a constraint and the constraint provider responsible for that constraint"""

    constraint_id: EntityID
    """ID of constraint to which this association pertains"""

    constraint_provider_id: str
    """Unique identifier of the constraint provider responsible for the constraint"""


class GeoZoneAdditionalProperties(ImplicitDict):
    """Indicates that exemptions from the national or European regulations are allowed in the UAS Zone, that will be detailed via the "message" property."""



CodeZoneIdentifierType = str
"""a string of maximum 7 characters that uniquely identifies the area within a geographical scope.
NOTE (1): This shall not include the country identifier, which is a separate attribute of the UAS Zone.
NOTE (2): The length of this data type is limited to 7 characters for compatibility with ARINC 424 and AIXM, where an airspace designator may have maximum 10 characters. The 10 characters are the result of concatenating the UAS Zone attributes for country and identifier.
"""


CodeCountryISOType = str
"""A 3 letter identifier of a country or territory using the ISO 3166-1 alpha-3 standard.
NOTE: >-
   The ISO 3-letter country codes come with the following advantages:
      - allow to distinguish between remote territories and mainland
      - are unique, unlike the ICAO Country codes where the same State
        could have two or more codes
      - are also used in military standards, such as NATO STANAG 1059
        INT, which come with well document additions that might be also
        useful for UAS areas.
"""


class CodeZoneType(str, Enum):
    """A coded list of values which allows indicating that the definition of a UAS Zone is specifically customised for a particular UAS or operator."""

    COMMON = "COMMON"
    CUSTOMIZED = "CUSTOMIZED"
    PROHIBITED = "PROHIBITED"
    REQ_AUTHORISATION = "REQ_AUTHORISATION"
    CONDITIONAL = "CONDITIONAL"
    NO_RESTRICTION = "NO_RESTRICTION"


ConditionExpressionType = str
"""A coded expression that provides information about what is authorised / forbidden in a zone that has conditional access.
By difference with the “Message” field per zone, this coded expression is made to be interopreted by the UAS while the “Message” is to interpreted by the remote pilot.
NOTE: the maximum field length is 10 000 characters.
---------------------- Condition definition language ---------------- • A list of relevant characteristics (CHARTYPE) has first to be established per state, and their finite list of acceptable values (CHARVAL)
• Each chartype and charval fields are defined by a limited set of characters
• A public document shall give the definitions of each, and provide the reference to legal or technical characteristics implied
• The Geozone editor per state can use these characteristics, with the dedicated condition language defined below, to define exact conditions per zone
• Each UAS Geofencing function shall be loaded with the corresponding chararacteristic status of the UAS for the intended flight, so as to be able to apply the conditions , either to generate alerts or to limit the flight
• If the value of a given characteristic of the condition equation is not defined in the UAS, the UAS Geofencing function should inform the pilot in Geoawareness alerting or consider that the zone is forbidden, by default in automatic Geofencing.
A conditional expression shall be of the following type:
• The UAS is PERMITTED XOR PROHIBITED (exclusive choice) to fly in this zone at this time IF (Characteristic1) CHARTYPE1 = (Value1) CHARVAL1 AND
CHARTYPE 2 = CHARVAL 2 AND ... AND End IF
OR (...)
...
End OR
• Only the fields in bold need to be edited in the character string, separated by”/”. Others are implicit.
Examples of CHARTYPE and CHARVALUE:
• CHARTYPE: operator type; Acceptable CHARVAL values: Military/Police/Firefighting
• CHARTYPE: Operator ID (registration number); Acceptable CHARVAL values: as per registration format
• CHARTYPE: Operation type: A1 as per EASA Open Types or S1 (National standard Scenario 1), STS01 (EASA Specific standard scenario) or ...
• CHARTYPE: UTM operation type: Planned/Unplanned,
• CHARTYPE: passengers on board: yes /no Note that it is possible in each national catalog of chartype and charval items, to define complex categories of operation/drone /equipment. Example: In nation A, we may have a type “drone level” with values Low, Medium, High. Each level corresponds to a defined set of required UAS performance/operation features/ operator qualification etc. This avoids to code a complex combination in the geozone database. This conditional expression can also be used to code a prohibition of image capture in a zone.
Example: PERMITTED/IMAGE CAPTURE=NO/NOISE

  CLASS=A/OR/OPERATOR=POLICE

Meaning: >-
   the fight is permitted in this zone at that time if No image is
captured (removed or deactivated) and if noise class = class A (following a known classification) or if the UAS operator is the Police
"""


CodeRestrictionType = str
"""An indication if flying in the zone is conditional, forbidden or unrestricted."""


class CodeZoneReasonType(str, Enum):
    """A coded indication of a reason that justifies the existence of an UAS Zone"""

    AIR_TRAFFIC = "AIR_TRAFFIC"
    SENSITIVE = "SENSITIVE"
    PRIVACY = "PRIVACY"
    POPULATION = "POPULATION"
    NATURE = "NATURE"
    NOISE = "NOISE"
    FOREIGN_TERRITORY = "FOREIGN_TERRITORY"
    EMERGENCY = "EMERGENCY"
    OTHER = "OTHER"


CodeUSpaceClassType = str
"""A coded identifier for a category or class of the zone applying a "USpace concept".
NOTE: >-
   In the current model version, there is no specific list of values.
For example, the “X”, “Y”, “Z” types of zones as per SESAR JU Corus project on USpace concept of operation could be used in a future version. Until a precise list of values is defined, this data type will be considered as string of characters of maximum 100 characters.
"""


CodeYesNoType = str
"""A coded value that indicates a choice between a positive (yes) or a negative (no) applicability.

Acceptable values:
* True
* False
"""


class CodeAuthorityRole(str, Enum):
    """A coded list of values indicating the role that an authority has in relation with the UAS zone."""

    AUTHORIZATION = "AUTHORIZATION"
    NOTIFICATION = "NOTIFICATION"
    INFORMATION = "INFORMATION"


TextShortType = str
"""A free text with a maximum length of 200 characters"""


class Polygon(ImplicitDict):
    """An enclosed area on the earth. The bounding edges of this polygon are defined to be the shortest paths between connected vertices.  This means, for instance, that the edge between two points both defined at a particular latitude is not generally contained at that latitude. The winding order must be interpreted as the order which produces the smaller area. The path between two vertices is defined to be the shortest possible path between those vertices. Edges may not cross. Vertices may not be duplicated.  In particular, the final polygon vertex must not be identical to the first vertex."""

    vertices: List[LatLngPoint]


class Volume3D(ImplicitDict):
    """A three-dimensional geographic volume consisting of a vertically-extruded shape. Exactly one outline must be specified."""

    outline_circle: Optional[Circle]
    """A circular geographic shape on the surface of the earth."""

    outline_polygon: Optional[Polygon]
    """A polygonal geographic shape on the surface of the earth."""

    altitude_lower: Optional[Altitude]
    """Minimum bounding altitude of this volume. Must be less than altitude_upper, if specified."""

    altitude_upper: Optional[Altitude]
    """Maximum bounding altitude of this volume. Must be greater than altitude_lower, if specified."""


class Volume4D(ImplicitDict):
    """Contiguous block of geographic spacetime."""

    volume: Volume3D

    time_start: Optional[Time]
    """Beginning time of this volume. Must be before time_end."""

    time_end: Optional[Time]
    """End time of this volume. Must be after time_start."""


class SubscriptionState(ImplicitDict):
    """State of subscription which is causing a notification to be sent."""

    subscription_id: SubscriptionID

    notification_index: SubscriptionNotificationIndex


class QuerySubscriptionParameters(ImplicitDict):
    """Parameters for a request to find subscriptions matching the provided criteria."""

    area_of_interest: Optional[Volume4D]


SubscriptionUssBaseURL = UssBaseURL


class OperationalIntentReference(ImplicitDict):
    """The high-level information of a planned or active operational intent with the URL of a USS to query for details.  Note: 'ovn' is returned ONLY to the USS that created the operational intent but NEVER to other USS instances."""

    id: EntityID

    manager: str
    """Created by the DSS based on creating client's ID (via access token).  Used internal to the DSS for restricting mutation and deletion operations to manager.  Used by USSs to reject operational intent update notifications originating from a USS that does not manage the operational intent."""

    uss_availability: UssAvailabilityState

    version: int
    """Numeric version of this operational intent which increments upon each change in the operational intent, regardless of whether any field of the operational intent reference changes.  A USS with the details of this operational intent when it was at a particular version does not need to retrieve the details again until the version changes."""

    state: OperationalIntentState

    ovn: Optional[EntityOVN]
    """Opaque version number of this operational intent.  Populated only when the OperationalIntentReference is managed by the USS retrieving or providing it.  Not populated when the OperationalIntentReference is not managed by the USS retrieving or providing it (instead, the USS must obtain the OVN from the details retrieved from the managing USS)."""

    time_start: Time
    """Beginning time of operational intent."""

    time_end: Time
    """End time of operational intent."""

    uss_base_url: OperationalIntentUssBaseURL

    subscription_id: SubscriptionID
    """The ID of the subscription that is ensuring the operational intent manager receives relevant airspace updates."""


class ImplicitSubscriptionParameters(ImplicitDict):
    """Information necessary to create a subscription to serve a single operational intent's notification needs."""

    uss_base_url: SubscriptionUssBaseURL
    """The base URL of a USS implementation of the parts of the USS-USS API necessary for receiving the notifications that the operational intent must be aware of.  This includes, at least, notifications for relevant changes in operational intents."""

    notify_for_constraints: Optional[bool] = False
    """True if this operational intent's subscription should trigger notifications when constraints change. Otherwise, changes in constraints should not trigger notifications.  The scope utm.constraint_processing is required to set this flag true, and a USS performing the constraint processing role should set this flag true."""


class GetOperationalIntentReferenceResponse(ImplicitDict):
    """Response to DSS request for the OperationalIntentReference with the given ID."""

    operational_intent_reference: OperationalIntentReference


class QueryOperationalIntentReferenceParameters(ImplicitDict):
    """Parameters for a request to find OperationalIntentReferences matching the provided criteria."""

    area_of_interest: Optional[Volume4D]


class QueryOperationalIntentReferenceResponse(ImplicitDict):
    """Response to DSS query for OperationalIntentReferences in an area of interest."""

    operational_intent_references: List[OperationalIntentReference] = []
    """OperationalIntentReferences in the area of interest."""


class ConstraintReference(ImplicitDict):
    """A ConstraintReference (area in which a constraint is present, along with other high-level information, but no details).  The DSS reports only these references and clients must exchange details and additional information peer-to-peer."""

    id: EntityID

    manager: str
    """Created by the DSS based on creating client's ID (via access token).  Used internal to the DSS for restricting mutation and deletion operations to manager.  Used by USSs to reject constraint update notifications originating from a USS that does not manage the constraint."""

    uss_availability: UssAvailabilityState

    version: int
    """Numeric version of this constraint which increments upon each change in the constraint, regardless of whether any field of the constraint reference changes.  A USS with the details of this constraint when it was at a particular version does not need to retrieve the details again until the version changes."""

    ovn: Optional[EntityOVN]
    """Opaque version number of this constraint.  Populated only when the ConstraintReference is managed by the USS retrieving or providing it.  Not populated when the ConstraintReference is not managed by the USS retrieving or providing it (instead, the USS must obtain the OVN from the details retrieved from the managing USS)."""

    time_start: Time

    time_end: Time

    uss_base_url: ConstraintUssBaseURL


class PutConstraintReferenceParameters(ImplicitDict):
    """Parameters for a request to create/update a ConstraintReference in the DSS."""

    extents: List[Volume4D]
    """Spacetime extents that bound this constraint.
    The end time may not be in the past.
    All volumes of the constraint must be encompassed in these extents. However, these extents do not need to match the precise volumes of the constraint; a single bounding extent may be provided instead, for instance.
    """

    uss_base_url: ConstraintUssBaseURL


class GetConstraintReferenceResponse(ImplicitDict):
    """Response to DSS request for the ConstraintReference with the given ID."""

    constraint_reference: ConstraintReference


class QueryConstraintReferenceParameters(ImplicitDict):
    """Parameters for a request to find ConstraintReferences matching the provided criteria."""

    area_of_interest: Optional[Volume4D]


class QueryConstraintReferencesResponse(ImplicitDict):
    """Response to DSS query for ConstraintReferences in an area of interest."""

    constraint_references: List[ConstraintReference] = []
    """ConstraintReferences in the area of interest."""


class AirspaceConflictResponse(ImplicitDict):
    """Data provided when an airspace conflict was encountered."""

    message: Optional[str]
    """Human-readable message indicating what error occurred and/or why."""

    missing_operational_intents: Optional[List[OperationalIntentReference]] = []
    """List of operational intent references for which current proof of knowledge was not provided.  If this field is present and contains elements, the calling USS should query the details URLs for these operational intents to obtain their details and correct OVNs.  The OVNs can be used to update the key, at which point the USS may retry this call."""

    missing_constraints: Optional[List[ConstraintReference]] = []
    """List of constraint references for which current proof of knowledge was not provided.  If this field is present and contains elements, the calling USS should query the details URLs for these constraints to obtain their details and correct OVNs.  The OVNs can be used to update the key, at which point the USS may retry this call."""


class OperationalIntentDetails(ImplicitDict):
    """Details of a UTM operational intent. Note that this data is not stored in the DSS; only with the clients."""

    volumes: Optional[List[Volume4D]] = []
    """Volumes that wholly contain the operational intent while being as small as practical.
    Start and end times, as well as lower and upper altitudes, are required for each volume. The end time may not be in the past.
    Required with at least one item when the operational intent is Accepted, Activated, or Nonconforming.
    May not contain any items when the operational intent is Contingent.
    """

    off_nominal_volumes: Optional[List[Volume4D]] = []
    """Volumes that contain the anticipated area of non-conformance while the aircraft is in the Nonconforming or Contingent states.
    Start and end times, as well as lower and upper altitudes, are required for each volume. The end time may not be in the past.
    Required with at least one item when the operational intent is Nonconforming or Contingent.
    May not contain any items when the operational intent is Accepted or Activated.
    """

    priority: Optional[Priority]


class OperationalIntent(ImplicitDict):
    """Full description of a UTM operational intent."""

    reference: OperationalIntentReference

    details: OperationalIntentDetails


class PutOperationalIntentDetailsParameters(ImplicitDict):
    """Parameters of a message informing of detailed information for a peer operational intent. Pushed (by a client, not the DSS) directly to clients with subscriptions when another client makes a change to airspace within a cell with a subscription."""

    operational_intent_id: EntityID
    """ID of operational intent that has changed."""

    operational_intent: Optional[OperationalIntent]
    """Full information about the operational intent that has changed.  If this field is omitted, the operational intent was deleted.  The `ovn` field in the nested `reference` must be populated."""

    subscriptions: List[SubscriptionState]
    """Subscription(s) prompting this notification."""


class GetOperationalIntentDetailsResponse(ImplicitDict):
    """Response to peer request for the details of operational intent with the given ID."""

    operational_intent: OperationalIntent


class VehicleTelemetry(ImplicitDict):
    """Vehicle position, altitude, and velocity."""

    time_measured: Time

    position: Optional[Position]

    velocity: Optional[Velocity]


class UssAvailabilityStatus(ImplicitDict):
    uss: str
    """Client ID (matching their `sub` in access tokens) of the USS to which this availability applies."""

    availability: UssAvailabilityState


class UssAvailabilityStatusResponse(ImplicitDict):
    version: str
    """Current version of USS's availability.  Used to change USS's availability."""

    status: UssAvailabilityStatus


class PositionRecord(ImplicitDict):
    """A record of vehicle telemetry information received by this USS (typically for conformance monitoring)."""

    time_received: Time
    """Time that this position data was received by the USS"""

    telemetry: VehicleTelemetry


class OperationalIntentPositions(ImplicitDict):
    """A record of position data gathered through the course of an operational intent"""

    positions: Optional[List[PositionRecord]] = []

    operational_intent_id: EntityID
    """ID of the operational intent associated with `positions`"""


class Authority(ImplicitDict):
    """A relevant authority that is in charge for authorising, being notified or providing information for UAS operations in the UAS zone.
    Rule: >-
       at least one of the following shall be specified - siteURL, email,
    phone.
    """

    name: Optional[TextShortType]
    """The official name of a public or private authority"""

    service: Optional[TextShortType]
    """The name of a specific department or service within the organisation"""

    contact_name: Optional[TextShortType]
    """The name or role of a specific person that needs to be contacted within the organisation"""

    site_url: Optional[TextShortType]
    """The URL of the public internet site through which the organisation may be contacted
    Note: in the data coding format, this might be further constrained in order to ensure a valid URL format.
    """

    email: Optional[TextShortType]
    """The e-mail address by which the organisation may be contacted.
    Note: in the data coding format, this might be further constrained in order to ensure a valid e-mail format.
    """

    phone: Optional[TextShortType]
    """A phone number at which the organisation may be contacted"""

    purpose: Optional[CodeAuthorityRole]
    """The role of the Authority in relation with the zone."""

    interval_before: Optional[str]
    """The minimal time interval required between notification or authorization request and starting to operate in the zone, in the format PnnDTnnHnnM (ISO 8601)."""


class SubscriberToNotify(ImplicitDict):
    """Subscriber to notify of a change in the airspace.  This is provided by the DSS to a client changing the airspace, and it is the responsibility of that client to send a notification to the specified USS according to the change made to the airspace."""

    subscriptions: List[SubscriptionState]
    """Subscription(s) prompting this notification."""

    uss_base_url: SubscriptionUssBaseURL


class Subscription(ImplicitDict):
    """Specification of a geographic area that a client is interested in on an ongoing basis (e.g., "planning area")."""

    id: SubscriptionID

    version: str
    """Version of the subscription that the DSS changes every time a USS changes the subscription.  The DSS incrementing the notification_index does not constitute a change that triggers a new version.  A USS must specify this version when modifying an existing subscription to ensure consistency in read-modify-write operations and distributed systems."""

    notification_index: SubscriptionNotificationIndex

    time_start: Optional[Time]
    """If set, this subscription will not receive notifications involving airspace changes entirely before this time."""

    time_end: Optional[Time]
    """If set, this subscription will not receive notifications involving airspace changes entirely after this time."""

    uss_base_url: SubscriptionUssBaseURL

    notify_for_operational_intents: Optional[bool] = False
    """If true, trigger notifications when operational intents are created, updated, or deleted.  Otherwise, changes in operational intents should not trigger notifications.  The scope utm.strategic_coordination is required to set this flag true."""

    notify_for_constraints: Optional[bool] = False
    """If true, trigger notifications when constraints are created, updated, or deleted.  Otherwise, changes in constraints should not trigger notifications.  The scope utm.constraint_processing is required to set this flag true."""

    implicit_subscription: Optional[bool] = False
    """True if this subscription was implicitly created by the DSS via the creation of an operational intent, and should therefore be deleted by the DSS when that operational intent is deleted."""

    dependent_operational_intents: Optional[List[EntityID]] = []
    """List of IDs for operational intents that are dependent on this subscription."""


class QuerySubscriptionsResponse(ImplicitDict):
    """Response to DSS query for subscriptions in a particular geographic area."""

    subscriptions: List[Subscription] = []
    """Subscriptions that overlap the specified geographic area."""


class GetSubscriptionResponse(ImplicitDict):
    """Response to DSS request for the subscription with the given id."""

    subscription: Subscription


class PutSubscriptionParameters(ImplicitDict):
    """Parameters for a request to create/update a subscription in the DSS.  At least one form of notifications must be requested."""

    extents: Volume4D
    """Spacetime extents of the volume to subscribe to.
    This subscription will automatically be deleted after its end time if it has not been refreshed by then. If end time is not specified, the value will be chosen automatically by the DSS. If start time is not specified, it will default to the time the request is processed. The end time may not be in the past.
    Note that some Entities triggering notifications may lie entirely outside the requested area.
    """

    uss_base_url: SubscriptionUssBaseURL

    notify_for_operational_intents: Optional[bool] = False
    """If true, trigger notifications when operational intents are created, updated, or deleted.  Otherwise, changes in operational intents should not trigger notifications.  The scope utm.strategic_coordination is required to set this flag true."""

    notify_for_constraints: Optional[bool] = False
    """If true, trigger notifications when constraints are created, updated, or deleted.  Otherwise, changes in constraints should not trigger notifications.  The scope utm.constraint_processing is required to set this flag true."""


class PutSubscriptionResponse(ImplicitDict):
    """Response for a request to create or update a subscription."""

    subscription: Subscription

    operational_intent_references: Optional[List[OperationalIntentReference]] = []
    """Operational intents in or near the subscription area at the time of creation/update, if `notify_for_operational_intents` is true."""

    constraint_references: Optional[List[ConstraintReference]] = []
    """Constraints in or near the subscription area at the time of creation/update, if `notify_for_constraints` is true."""


class DeleteSubscriptionResponse(ImplicitDict):
    """Response for a successful request to delete a subscription."""

    subscription: Subscription


class PutOperationalIntentReferenceParameters(ImplicitDict):
    """Parameters for a request to create an OperationalIntentReference in the DSS. A subscription to changes overlapping this volume may be implicitly created, but this can be overridden by providing the (optional) 'subscription_id' to use. Note: The implicit subscription is managed by the DSS, not the USS."""

    extents: List[Volume4D]
    """Spacetime extents that bound this operational intent.
    Start and end times, as well as lower and upper altitudes, are required for each volume. The end time may not be in the past. All volumes, both nominal and off-nominal, must be encompassed in these extents. However, these extents do not need to match the precise volumes of the operational intent; a single bounding extent may be provided instead, for instance.
    """

    key: Optional[Key]
    """Proof that the USS creating or mutating this operational intent was aware of the current state of the airspace, with the expectation that this operational intent is therefore deconflicted from all relevant features in the airspace.  This field is not required when declaring an operational intent Nonconforming or Contingent, or when there are no relevant Entities in the airspace, but is otherwise required. OVNs for constraints are required if and only if the USS managing this operational intent is performing the constraint processing role, which is indicated by whether the subscription associated with this operational intent triggers notifications for constraints.  The key does not need to contain the OVN for the operational intent being updated."""

    state: OperationalIntentState

    uss_base_url: OperationalIntentUssBaseURL

    subscription_id: Optional[EntityID]
    """The ID of an existing subscription that the USS will use to keep the operator informed about updates to relevant airspace information. If this field is not provided when the operational intent is in the Activated, Nonconforming, or Contingent state, then the `new_subscription` field must be provided in order to provide notification capability for the operational intent.  The subscription specified by this ID must cover at least the area over which this operational intent is conducted, and it must provide notifications for operational intents."""

    new_subscription: Optional[ImplicitSubscriptionParameters]
    """If an existing subscription is not specified in `subscription_id`, and the operational intent is in the Activated, Nonconforming, or Contingent state, then this field must be populated.  When this field is populated, an implicit subscription will be created and associated with this operational intent, and will generally be deleted automatically upon the deletion of this operational intent."""


class ChangeOperationalIntentReferenceResponse(ImplicitDict):
    """Response to a request to create, update, or delete an OperationalIntentReference in the DSS."""

    subscribers: List[SubscriberToNotify] = []
    """DSS subscribers that this client now has the obligation to notify of the operational intent changes just made.  This client must call POST for each provided URL according to the USS-USS `/uss/v1/operational_intents` path API.  The client's own subscriptions will also be included in this list."""

    operational_intent_reference: OperationalIntentReference


class ChangeConstraintReferenceResponse(ImplicitDict):
    """Response to a request to create, update, or delete a ConstraintReference. in the DSS."""

    subscribers: List[SubscriberToNotify] = []
    """DSS subscribers that this client now has the obligation to notify of the constraint changes just made.  This client must call POST for each provided URL according to the USS-USS `/uss/v1/constraints` path API.  The client's own subscriptions will also be included in this list."""

    constraint_reference: ConstraintReference


class GetOperationalIntentTelemetryResponse(ImplicitDict):
    """Response to a peer request for telemetry of an off-nominal operational intent."""

    operational_intent_id: EntityID
    """ID of the operational intent which the vehicle reporting telemetry is flying."""

    telemetry: Optional[VehicleTelemetry]

    next_telemetry_opportunity: Optional[Time]
    """The next telemetry similar to this telemetry is not expected to be available until at or after this time, so the polling USS should generally not poll the endpoint providing this response data again until at or after that time.  If this field is omitted, then there is no current expectation of new telemetry becoming available."""


class USSLogSet(ImplicitDict):
    """The set of log data fulfilling this standard's Logging requirements."""

    messages: Optional[List[ExchangeRecord]] = []
    """Outgoing messages sent to other USSs and the DSS, and incoming messages received from other USSs, including instances where an expected response to a request is not received."""

    operator_notifications: Optional[List[UserNotificationRecord]] = []
    """Instances of operator notifications as specifically required within this standard."""

    operator_inputs: Optional[List[UserInputRecord]] = []
    """Instances of operator input as specifically required within this standard."""

    operator_associations: Optional[List[OperatorAssociation]] = []
    """For a USS that manages operational intents, associations of an operator with operational intents that transitioned to the Accepted state."""

    planning_attempts: Optional[List[PlanningRecord]] = []
    """For a USS that manages operational intents, instances where an operational intent could not be planned or replanned due to conflicts with other operational intents or constraints."""

    operational_intent_positions: Optional[List[OperationalIntentPositions]] = []
    """For a USS performing conformance monitoring, all position data used for conformance monitoring that is ingested from the UA."""

    constraint_provider_associations: Optional[List[ConstraintProviderAssociation]] = []
    """For a USS that performs constraint management, associations of an authorized constraint provider with all constraints that transition to the valid state."""


class GeoZone(ImplicitDict):
    """An airspace of defined dimensions, above the land areas or territorial waters of a State, within which a particular restriction or condition for UAS flights applies."""

    identifier: CodeZoneIdentifierType
    """A string of characters that uniquely identifies the UAS Zone within the State/Territory identified by the country attribute.
    Note - The UAS Zone is uniquely identified worldwide by the combination of the country and the identifier attributes
    """

    country: CodeCountryISOType
    """The State that has the authority to declare the zone.
    Note - There will be no Zone belonging to two States. Not necessary to code the information that two zones are "in neighboring States" or "related".
    """

    zone_authority: List[Authority]

    name: Optional[TextShortType]
    """A free text name by which the zone may be known by the public or by the UAS community."""

    type: CodeZoneType
    """An indication whether the Zone is provided with its common definition or with a customised definition, for a particular user."""

    restriction: CodeRestrictionType
    """An indication if flying in the zone is conditional, forbidden or unrestricted."""

    restriction_conditions: Optional[List[ConditionExpressionType]]
    """An indication of the conditions under which the zone can be used"""

    region: Optional[int]
    """Where applicable, identifies a region inside a State where the UAS Zone is located.
    Note 1) identified with a digit between 0-65535 (16 bit), corresponding to a list of regions pre-defined for each State.
    Note 2) this attribute is intended to facilitate extracting sub-sets of data, for specific regions
    """

    reason: Optional[List[CodeZoneReasonType]]
    """A coded indication for the reason that led to the establishment of the zone."""

    other_reason_info: Optional[str]
    """A free text description of the reason that led to the establishment of the zone, when not covered by a pre-defined coded value."""

    regulation_exemption: Optional[CodeYesNoType]
    """This is an extension point. It allows adding additional attributes of national interest through this element."""

    u_space_class: Optional[CodeUSpaceClassType]
    """A code that identifies the category or class of the zone applying a "USpace concept".
    Note: Two (draft) classifications exist, one from Eurocontrol and one from CORUS. Therefore, two instances of this attribute are expected, one from each sub-list. This might be later replaced with separate attributes and separate lists of values.
    """

    message: Optional[TextShortType]
    """A message to be displayed to the user of the zone, typically on the RPS for the Remote Pilot, to make him/her aware about specific information associated with the zone (typically when it is not only a restriction to fly in the zone, thus not only an alert or an automatic limitation, for example : “image capture prohibited in this zone”, “frequent strong winds in this zone”, “no landing or take-off in this zone”). This message is also used to indicate exemptions from regulation in a zone (see below). Several information can be grouped in a message, separated by a “/”."""

    additional_properties: Optional[GeoZoneAdditionalProperties]
    """Indicates that exemptions from the national or European regulations are allowed in the UAS Zone, that will be detailed via the "message" property."""


class ConstraintDetails(ImplicitDict):
    """Details of a UTM constraint. Note that this data is not stored in the DSS; only with the clients."""

    volumes: List[Volume4D]
    """Volumes that wholly contain the constraint while being as small as practical.
    The end time may not be in the past.
    """

    type: Optional[str]
    """Type of airspace feature this constraint represents."""

    geozone: Optional[GeoZone]
    """If this constraint is an ED-269 compliant geo zone, the details about that geo zone."""


class Constraint(ImplicitDict):
    """Full specification of a UTM constraint."""

    reference: ConstraintReference

    details: ConstraintDetails


class PutConstraintDetailsParameters(ImplicitDict):
    """Parameters of a message informing of new full information for a constraint.  Pushed (by a client, not the DSS) directly to clients with subscriptions when another client makes a change to airspace within a cell with a subscription."""

    constraint_id: EntityID
    """ID of constraint that has changed."""

    constraint: Optional[Constraint]
    """Full information about the constraint that has changed.  If this field is omitted, the constraint was deleted.  The `ovn` field in the nested `reference` must be populated."""

    subscriptions: List[SubscriptionState]
    """Subscription(s) prompting this notification."""


class GetConstraintDetailsResponse(ImplicitDict):
    """Response to peer request for the details of operational intent with the given ID."""

    constraint: Constraint


class OperationID(str, Enum):
    QueryOperationalIntentReferences = "queryOperationalIntentReferences"
    GetOperationalIntentReference = "getOperationalIntentReference"
    CreateOperationalIntentReference = "createOperationalIntentReference"
    UpdateOperationalIntentReference = "updateOperationalIntentReference"
    DeleteOperationalIntentReference = "deleteOperationalIntentReference"
    QueryConstraintReferences = "queryConstraintReferences"
    GetConstraintReference = "getConstraintReference"
    CreateConstraintReference = "createConstraintReference"
    UpdateConstraintReference = "updateConstraintReference"
    DeleteConstraintReference = "deleteConstraintReference"
    QuerySubscriptions = "querySubscriptions"
    GetSubscription = "getSubscription"
    CreateSubscription = "createSubscription"
    UpdateSubscription = "updateSubscription"
    DeleteSubscription = "deleteSubscription"
    MakeDssReport = "makeDssReport"
    SetUssAvailability = "setUssAvailability"
    GetUssAvailability = "getUssAvailability"
    GetOperationalIntentDetails = "getOperationalIntentDetails"
    GetOperationalIntentTelemetry = "getOperationalIntentTelemetry"
    NotifyOperationalIntentDetailsChanged = "notifyOperationalIntentDetailsChanged"
    GetConstraintDetails = "getConstraintDetails"
    NotifyConstraintDetailsChanged = "notifyConstraintDetailsChanged"
    MakeUssReport = "makeUssReport"
    GetLogSet = "getLogSet"


OPERATIONS: Dict[OperationID, Operation] = {
    OperationID.QueryOperationalIntentReferences: Operation(
        id="queryOperationalIntentReferences",
        path="/dss/v1/operational_intent_references/query",
        verb="POST",
        request_body_type=QueryOperationalIntentReferenceParameters,
        response_body_type={
            200: QueryOperationalIntentReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            413: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.GetOperationalIntentReference: Operation(
        id="getOperationalIntentReference",
        path="/dss/v1/operational_intent_references/{entityid}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetOperationalIntentReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.CreateOperationalIntentReference: Operation(
        id="createOperationalIntentReference",
        path="/dss/v1/operational_intent_references/{entityid}",
        verb="PUT",
        request_body_type=PutOperationalIntentReferenceParameters,
        response_body_type={
            201: ChangeOperationalIntentReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: AirspaceConflictResponse,
            412: ErrorResponse,
            413: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.UpdateOperationalIntentReference: Operation(
        id="updateOperationalIntentReference",
        path="/dss/v1/operational_intent_references/{entityid}/{ovn}",
        verb="PUT",
        request_body_type=PutOperationalIntentReferenceParameters,
        response_body_type={
            200: ChangeOperationalIntentReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: AirspaceConflictResponse,
            412: ErrorResponse,
            413: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.DeleteOperationalIntentReference: Operation(
        id="deleteOperationalIntentReference",
        path="/dss/v1/operational_intent_references/{entityid}/{ovn}",
        verb="DELETE",
        request_body_type=None,
        response_body_type={
            200: ChangeOperationalIntentReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            409: ErrorResponse,
            412: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.QueryConstraintReferences: Operation(
        id="queryConstraintReferences",
        path="/dss/v1/constraint_references/query",
        verb="POST",
        request_body_type=QueryConstraintReferenceParameters,
        response_body_type={
            200: QueryConstraintReferencesResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            413: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.GetConstraintReference: Operation(
        id="getConstraintReference",
        path="/dss/v1/constraint_references/{entityid}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetConstraintReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.CreateConstraintReference: Operation(
        id="createConstraintReference",
        path="/dss/v1/constraint_references/{entityid}",
        verb="PUT",
        request_body_type=PutConstraintReferenceParameters,
        response_body_type={
            201: ChangeConstraintReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            413: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.UpdateConstraintReference: Operation(
        id="updateConstraintReference",
        path="/dss/v1/constraint_references/{entityid}/{ovn}",
        verb="PUT",
        request_body_type=PutConstraintReferenceParameters,
        response_body_type={
            200: ChangeConstraintReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            413: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.DeleteConstraintReference: Operation(
        id="deleteConstraintReference",
        path="/dss/v1/constraint_references/{entityid}/{ovn}",
        verb="DELETE",
        request_body_type=None,
        response_body_type={
            200: ChangeConstraintReferenceResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            409: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.QuerySubscriptions: Operation(
        id="querySubscriptions",
        path="/dss/v1/subscriptions/query",
        verb="POST",
        request_body_type=QuerySubscriptionParameters,
        response_body_type={
            200: QuerySubscriptionsResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            413: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.GetSubscription: Operation(
        id="getSubscription",
        path="/dss/v1/subscriptions/{subscriptionid}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetSubscriptionResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.CreateSubscription: Operation(
        id="createSubscription",
        path="/dss/v1/subscriptions/{subscriptionid}",
        verb="PUT",
        request_body_type=PutSubscriptionParameters,
        response_body_type={
            200: PutSubscriptionResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.UpdateSubscription: Operation(
        id="updateSubscription",
        path="/dss/v1/subscriptions/{subscriptionid}/{version}",
        verb="PUT",
        request_body_type=PutSubscriptionParameters,
        response_body_type={
            200: PutSubscriptionResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.DeleteSubscription: Operation(
        id="deleteSubscription",
        path="/dss/v1/subscriptions/{subscriptionid}/{version}",
        verb="DELETE",
        request_body_type=None,
        response_body_type={
            200: DeleteSubscriptionResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            409: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.MakeDssReport: Operation(
        id="makeDssReport",
        path="/dss/v1/reports",
        verb="POST",
        request_body_type=ErrorReport,
        response_body_type={
            201: ErrorReport,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.SetUssAvailability: Operation(
        id="setUssAvailability",
        path="/dss/v1/uss_availability/{uss_id}",
        verb="PUT",
        request_body_type=SetUssAvailabilityStatusParameters,
        response_body_type={
            200: UssAvailabilityStatusResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.GetUssAvailability: Operation(
        id="getUssAvailability",
        path="/dss/v1/uss_availability/{uss_id}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: UssAvailabilityStatusResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.GetOperationalIntentDetails: Operation(
        id="getOperationalIntentDetails",
        path="/uss/v1/operational_intents/{entityid}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetOperationalIntentDetailsResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.GetOperationalIntentTelemetry: Operation(
        id="getOperationalIntentTelemetry",
        path="/uss/v1/operational_intents/{entityid}/telemetry",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetOperationalIntentTelemetryResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            409: ErrorResponse,
            412: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.NotifyOperationalIntentDetailsChanged: Operation(
        id="notifyOperationalIntentDetailsChanged",
        path="/uss/v1/operational_intents",
        verb="POST",
        request_body_type=PutOperationalIntentDetailsParameters,
        response_body_type={
            204: None,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.GetConstraintDetails: Operation(
        id="getConstraintDetails",
        path="/uss/v1/constraints/{entityid}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetConstraintDetailsResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.NotifyConstraintDetailsChanged: Operation(
        id="notifyConstraintDetailsChanged",
        path="/uss/v1/constraints",
        verb="POST",
        request_body_type=PutConstraintDetailsParameters,
        response_body_type={
            204: None,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.MakeUssReport: Operation(
        id="makeUssReport",
        path="/uss/v1/reports",
        verb="POST",
        request_body_type=ErrorReport,
        response_body_type={
            201: ErrorReport,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.GetLogSet: Operation(
        id="getLogSet",
        path="/uss/v1/log_sets/{log_set_id}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: USSLogSet,
        }
    ),
}



from enum import Enum


class Scope(str, Enum):
    Read = "dss.read.identification_service_areas"
    Write = "dss.write.identification_service_areas"


NetMinUasLocRefreshFrequencyHz = 1
NetMinUasLocRefreshPercentage = 20
NetMaxDisplayAreaDiagonalKm = 3.6
NetSpDataResponseTime95thPercentileSeconds = 1
NetSpDataResponseTime99thPercentileSeconds = 3
NetMaxNearRealTimeDataPeriodSeconds = 60
NetDpMaxDataRetentionPeriodSeconds = 86400
NetDpInitResponse95thPercentileSeconds = 6
NetDpInitResponse99thPercentileSeconds = 18
NetDpDataResponse95thPercentileSeconds = 1
NetDpDataResponse99thPercentileSeconds = 3
NetMinSessionLengthSeconds = 5
NetDpDetailsResponse95thPercentileSeconds = 2
NetDpDetailsResponse99thPercentileSeconds = 6
NetDetailsMaxDisplayAreaDiagonalKm = 1
NetMinClusterSizePercent = 15
NetMinObfuscationDistanceM = 300
NetDSSMaxSubscriptionPerArea = 10
NetDSSMaxSubscriptionDurationHours = 24



"""Data types and operations from Discovery and Synchronization Service 1.0.0 OpenAPI"""

# This file is autogenerated; do not modify manually!

from __future__ import annotations

from enum import Enum
from typing import Dict, List, Optional

from uas_standards import Operation

from implicitdict import ImplicitDict, StringBasedDateTime


API_VERSION = "1.0.0"
"""Version of Discovery and Synchronization Service OpenAPI specification from which the objects in this package were generated."""

URL = str
"""Valid http or https URL."""


SubscriptionNotificationIndex = int
"""Tracks the notifications sent for a subscription so the subscriber can detect missed notifications more easily."""


UUIDv4 = str
"""UUID v4."""


Version = str
"""A version string used to reference an object at a particular point in time. Any updates to an object must contain the corresponding version to maintain idempotent updates."""


EntityUUID = UUIDv4


SubscriptionUUID = UUIDv4


RIDFlightID = str
"""ID, unique to a remote ID service provider, which identifies a particular flight for which the remote ID service provider is providing remote ID services.

The following characters are not allowed: \\0 \\t \\r \\n # % / : ? @ [ \\ ]
"""


class RIDAuthData(ImplicitDict):
    """Additional authentication data."""

    format: str
    """Format of additional authentication data."""

    data: str
    """Authentication data in form specified by `format`."""


class HorizontalAccuracy(str, Enum):
    """This is the NACp enumeration from ADS-B, plus 1m for a more complete range for UAs.

    `HAUnknown`: Unknown horizontal accuracy

    `HA10NMPlus`: > 10NM (18.52km)

    `HA10NM`: < 10NM (18.52km)

    `HA4NM`: < 4NM (7.408km)

    `HA2NM`: < 2NM (3.704km)

    `HA1NM`: < 1NM (1852m)

    `HA05NM`: < 0.5NM (926m)

    `HA03NM`: < 0.3NM (555.6m)

    `HA01NM`: < 0.1NM (185.2m)

    `HA005NM`: < 0.05NM (92.6m)

    `HA30m`: < 30m

    `HA10m`: < 10m

    `HA3m`: < 3m

    `HA1m`: < 1m
    """

    HAUnknown = "HAUnknown"
    HA10NMPlus = "HA10NMPlus"
    HA10NM = "HA10NM"
    HA4NM = "HA4NM"
    HA2NM = "HA2NM"
    HA1NM = "HA1NM"
    HA05NM = "HA05NM"
    HA03NM = "HA03NM"
    HA01NM = "HA01NM"
    HA005NM = "HA005NM"
    HA30m = "HA30m"
    HA10m = "HA10m"
    HA3m = "HA3m"
    HA1m = "HA1m"


class VerticalAccuracy(str, Enum):
    """This is the GVA enumeration from ADS-B, plus some finer values for UAs.

    `VAUnknown`: Unknown vertical accuracy

    `VA150mPlus`: > 150m

    `VA150m`: < 150m

    `VA45m`: < 45m

    `VA25m`: < 25m

    `VA10m`: < 10m

    `VA3m`: < 3m

    `VA1m`: < 1m
    """

    VAUnknown = "VAUnknown"
    VA150mPlus = "VA150mPlus"
    VA150m = "VA150m"
    VA45m = "VA45m"
    VA25m = "VA25m"
    VA10m = "VA10m"
    VA3m = "VA3m"
    VA1m = "VA1m"


class ErrorResponse(ImplicitDict):
    """Data provided when an off-nominal condition was encountered."""

    message: Optional[str]
    """Human-readable message indicating what error occurred and/or why."""


class SpeedAccuracy(str, Enum):
    """This is the same enumeration scale and values from ADS-B NACv.

    `SAUnknown`: Unknown speed accuracy

    `SA10mpsPlus`: > 10 m/s

    `SA10mps`: < 10 m/s

    `SA3mps`: < 3 m/s

    `SA1mps`: < 1 m/s

    `SA03mps`: < 0.3 m/s
    """

    SAUnknown = "SAUnknown"
    SA10mpsPlus = "SA10mpsPlus"
    SA10mps = "SA10mps"
    SA3mps = "SA3mps"
    SA1mps = "SA1mps"
    SA03mps = "SA03mps"


GeoPolygonString = str
"""Plain-string representation of a geographic polygon consisting of at least three geographic points describing a closed polygon on the earth.  Each point consists of latitude,longitude in degrees.  Points are also comma-delimited, so this parameter will look like `lat1,lng1,lat2,lng2,lat3,lng3,...`  Latitude values must fall in the range [-90, 90] and longitude values must fall in the range [-180, 180].

All of the requirements and clarifications for GeoPolygon apply to GeoPolygonString as well.
"""


class RIDHeightReference(str, Enum):
    """The reference datum above which the height is reported."""

    TakeoffLocation = "TakeoffLocation"
    GroundLevel = "GroundLevel"


class RIDHeight(ImplicitDict):
    """A relative altitude for the purposes of remote ID."""

    distance: float
    """Distance above reference datum.  This value is provided in meters and must have a minimum resolution of 1 meter."""

    reference: RIDHeightReference
    """The reference datum above which the height is reported."""


Latitude = float
"""Degrees of latitude north of the equator, with reference to the WGS84 ellipsoid."""


Longitude = float
"""Degrees of longitude east of the Prime Meridian, with reference to the WGS84 ellipsoid."""


class LatLngPoint(ImplicitDict):
    """Point on the earth's surface."""

    lng: Longitude

    lat: Latitude


Altitude = float
"""An altitude, in meters, above the WGS84 ellipsoid."""


class GeoPolygon(ImplicitDict):
    """An enclosed area on the earth.
    The bounding edges of this polygon shall be the shortest paths between connected vertices.  This means, for instance, that the edge between two points both defined at a particular latitude is not generally contained at that latitude.
    The winding order shall be interpreted as the order which produces the smaller area.
    The path between two vertices shall be the shortest possible path between those vertices.
    Edges may not cross.
    Vertices may not be duplicated.  In particular, the final polygon vertex shall not be identical to the first vertex.
    """

    vertices: List[LatLngPoint]


IdentificationServiceAreaURL = str
"""The URL at which notifications regarding Identification Service Areas may be delivered.  See the `/uss/identification_service_areas/{id}` path for specification of this endpoint."""


class SubscriptionCallbacks(ImplicitDict):
    """Endpoints that should be called when an applicable event occurs.  At least one field must be specified."""

    identification_service_area_url: Optional[IdentificationServiceAreaURL]
    """If specified, other clients will be instructed by the DSS to call this endpoint when an Identification Service Area relevant to this Subscription is created, modified, or deleted.  Must implement PUT and DELETE according to the `/uss/identification_service_areas/{id}` path API."""


class RIDOperationalStatus(str, Enum):
    """Indicates operational status of associated aircraft."""

    Undeclared = "Undeclared"
    Ground = "Ground"
    Airborne = "Airborne"


class RIDFlightDetails(ImplicitDict):
    """Details about a flight reported by a remote ID service provider.  At least one of the registration or serial fields must be filled if required by CAA."""

    id: str
    """ID for this flight, matching argument in request."""

    operator_id: Optional[str]
    """CAA-issued registration/license ID for the remote pilot or operator. """

    operator_location: Optional[LatLngPoint]
    """Location of party controlling the aircraft."""

    operation_description: Optional[str]
    """Free-text field that enables the operator to describe the purpose of a flight, if so desired."""

    auth_data: Optional[RIDAuthData]

    serial_number: Optional[str]
    """Can be specified when no registration ID exists and required by law in a region. This is expressed in the ANSI/CTA-2063-A Physical Serial Number format."""

    registration_number: Optional[str]
    """If a CAA provides a method of registering UAS, this number is provided by the CAA or its authorized representative.  Required when required by law in a region."""


class Subscription(ImplicitDict):
    """Specification of a geographic area that a client is interested in on an ongoing basis (e.g., “planning area”).  Internal to the DSS."""

    id: SubscriptionUUID
    """Unique identifier for this subscription."""

    callbacks: SubscriptionCallbacks

    owner: str
    """Assigned by the DSS based on creating client’s ID (via access token).  Used for restricting mutation and deletion operations to owner."""

    notification_index: SubscriptionNotificationIndex

    time_end: Optional[StringBasedDateTime]
    """If set, this subscription will be automatically removed after this time.  RFC 3339 format, per OpenAPI specification."""

    time_start: Optional[StringBasedDateTime]
    """If set, this Subscription will not generate any notifications before this time.  RFC 3339 format, per OpenAPI specification."""

    version: Version


class RIDAircraftType(str, Enum):
    """Type of aircraft for the purposes of remote ID.

    `VTOL` is a fixed wing aircraft that can take off vertically.  `Helicopter` includes multirotor.
    """

    NotDeclared = "NotDeclared"
    Aeroplane = "Aeroplane"
    Helicopter = "Helicopter"
    Gyroplane = "Gyroplane"
    VTOL = "VTOL"
    Ornithopter = "Ornithopter"
    Glider = "Glider"
    Kite = "Kite"
    FreeBalloon = "FreeBalloon"
    CaptiveBalloon = "CaptiveBalloon"
    Airship = "Airship"
    FreeFallOrParachute = "FreeFallOrParachute"
    Rocket = "Rocket"
    TetheredPoweredAircraft = "TetheredPoweredAircraft"
    GroundObstacle = "GroundObstacle"
    Other = "Other"


RIDFlightsURL = str
"""The URL at which the remote ID flights and their details may be retrieved.  See `/flights` and `/flights/{id}/details` paths for specification of this endpoint.
This URL is the base flights resource.  If this URL is specified as https://example.com/flights then the flight details for a particular {id} may be obtained at the URL https://example.com/flights/{id}/details.  This URL may not have a trailing / character.
"""


class Volume3D(ImplicitDict):
    """A three-dimensional geographic volume consisting of a vertically-extruded polygon."""

    footprint: GeoPolygon
    """Projection of this volume onto the earth's surface."""

    altitude_lo: Optional[Altitude]
    """Minimum bounding altitude of this volume."""

    altitude_hi: Optional[Altitude]
    """Maximum bounding altitude of this volume."""


class Volume4D(ImplicitDict):
    """Contiguous block of geographic spacetime."""

    spatial_volume: Volume3D
    """Constant spatial extent of this volume."""

    time_start: Optional[StringBasedDateTime]
    """Beginning time of this volume.  RFC 3339 format, per OpenAPI specification."""

    time_end: Optional[StringBasedDateTime]
    """End time of this volume.  RFC 3339 format, per OpenAPI specification."""


class GetSubscriptionResponse(ImplicitDict):
    """Response to DSS request for the subscription with the given id."""

    subscription: Subscription


class SearchSubscriptionsResponse(ImplicitDict):
    """Response to DSS query for subscriptions in a particular area."""

    subscriptions: List[Subscription]
    """Subscriptions that overlap the specified area."""


class SubscriptionState(ImplicitDict):
    """State of AreaSubscription which is causing a notification to be sent."""

    subscription_id: SubscriptionUUID

    notification_index: Optional[SubscriptionNotificationIndex]


class GetFlightDetailsResponse(ImplicitDict):
    """Response to remote ID provider query for details about a specific flight."""

    details: RIDFlightDetails


class DeleteSubscriptionResponse(ImplicitDict):
    """Response for a successful request to delete an Subscription."""

    subscription: Subscription
    """The Subscription which was deleted."""


class RIDAircraftPosition(ImplicitDict):
    """Position of an aircraft as reported for remote ID purposes."""

    lat: Latitude

    lng: Longitude

    alt: float
    """Geodetic altitude (NOT altitude above launch, altitude above ground, or EGM96): aircraft distance above the WGS84 ellipsoid as measured along a line that passes through the aircraft and is normal to the surface of the WGS84 ellipsoid.  This value is provided in meters and must have a minimum resolution of 1 meter."""

    accuracy_h: Optional[HorizontalAccuracy]
    """Horizontal error that is likely to be present in this reported position.  Required when `extrapolated` field is true and always in the entry for the current state."""

    accuracy_v: Optional[VerticalAccuracy]
    """Vertical error that is likely to be present in this reported position.  Required when `extrapolated` field is true and always in the entry for the current state."""

    extrapolated: Optional[bool]
    """True if this position was generated primarily by computation rather than primarily from a direct instrument measurement.  Assumed false if not specified."""

    pressure_altitude: Optional[float]
    """The uncorrected altitude (based on reference standard 29.92 inHg, 1013.25 mb) provides a reference for algorithms that utilize "altitude deltas" between aircraft.  This value is provided in meters and must have a minimum resolution of 1 meter."""


class SubscriberToNotify(ImplicitDict):
    """Subscriber to notify of a creation/change/deletion of a change in the airspace.  This is provided by the DSS to a client changing the airspace, and it is the responsibility of the client changing the airspace (they will receive a set of these notification requests) to send a notification to each specified `url`."""

    subscriptions: List[SubscriptionState]
    """Subscription(s) prompting this notification."""

    url: URL
    """The endpoint that the client mutating the airspace should provide the update to.  API depends on the DSS action taken that triggered this notification request."""


class RIDRecentAircraftPosition(ImplicitDict):
    time: StringBasedDateTime
    """Time at which this position applied.  RFC 3339 format, per OpenAPI specification."""

    position: RIDAircraftPosition


class CreateIdentificationServiceAreaParameters(ImplicitDict):
    """Parameters for a request to create an Identification Service Area in the DSS."""

    extents: Volume4D
    """The bounding spacetime extents of this Identification Service Area.  End time must be specified.  If start time is not specified, it will be set to the current time.  Start times in the past should be rejected by the DSS, except that it may adjust very recent start times to the current time.

    These extents should not reveal any sensitive information about the flight or flights within them.  This means, for instance, that extents should not tightly-wrap a flight path, nor should they generally be centered around the takeoff point of a single flight.
    """

    flights_url: RIDFlightsURL


class UpdateIdentificationServiceAreaParameters(ImplicitDict):
    """Parameters for a request to update an Identification Service Area in the DSS."""

    extents: Volume4D
    """The bounding spacetime extents of this Identification Service Area.  End time must be specified.  If start time is not specified, it will remain unchanged.  Start times in the past should be rejected by the DSS unless they are unchanged from the Identification Service Area's current start time.

    These extents should not reveal any sensitive information about the flight or flights within them.  This means, for instance, that extents should not tightly-wrap a flight path, nor should they generally be centered around the takeoff point of a single flight.
    """

    flights_url: RIDFlightsURL


class CreateSubscriptionParameters(ImplicitDict):
    """Parameters for a request to create a subscription in the DSS."""

    extents: Volume4D
    """The spacetime extents of the volume to subscribe to.

    This subscription will automatically be deleted after its end time if it has not been refreshed by then.  If end time is not specified, the value will be chosen automatically by the DSS.

    Note that some Entities triggering notifications may lie entirely outside the requested area.
    """

    callbacks: SubscriptionCallbacks


class UpdateSubscriptionParameters(ImplicitDict):
    """Parameters for a request to update a subscription in the DSS."""

    extents: Volume4D
    """The spacetime extents of the volume to subscribe to.

    This subscription will automatically be deleted after its end time if it has not been refreshed by then.  If end time is not specified, the value will be chosen automatically by the DSS.

    Note that some Entities triggering notifications may lie entirely outside the requested area.
    """

    callbacks: SubscriptionCallbacks


class IdentificationServiceArea(ImplicitDict):
    """An Identification Service Area (area in which remote ID services are being provided).  The DSS reports only these declarations and clients must exchange flight information peer-to-peer."""

    flights_url: RIDFlightsURL

    owner: str
    """Assigned by the DSS based on creating client’s ID (via access token).  Used for restricting mutation and deletion operations to owner."""

    time_start: StringBasedDateTime
    """Beginning time of service.  RFC 3339 format, per OpenAPI specification."""

    time_end: StringBasedDateTime
    """End time of service.  RFC 3339 format, per OpenAPI specification."""

    version: Version

    id: EntityUUID
    """Unique identifier for this Identification Service Area."""


class RIDAircraftState(ImplicitDict):
    """State of an aircraft for the purposes of remote ID."""

    timestamp: StringBasedDateTime
    """Time at which this state was valid.  This may be the time coming from the source, such as a GPS, or the time when the system computes the values using an algorithm such as an Extended Kalman Filter (EKF).  Timestamp must be expressed with a minimum resolution of 1/10th of a second.  RFC 3339 format, per OpenAPI specification."""

    timestamp_accuracy: float
    """Declaration of timestamp accuracy, which is the largest difference between Timestamp and true time of applicability for any of the following fields: Latitude, Longitude, Geodetic Altitude, Pressure Altitude of Position, Height. to determine time of applicability of the location data provided.  Expressed in seconds, precise to 1/10ths of seconds. The accuracy reflects the 95% uncertainty bound value for the timestamp."""

    operational_status: Optional[RIDOperationalStatus]

    position: RIDAircraftPosition

    track: float
    """Direction of flight expressed as a "True North-based" ground track angle.  This value is provided in degrees East of North with a minimum resolution of 1 degree."""

    speed: float
    """Ground speed of flight in meters per second."""

    speed_accuracy: SpeedAccuracy
    """Accuracy of horizontal ground speed."""

    vertical_speed: float
    """Speed up (vertically) WGS84-HAE, m/s."""

    height: Optional[RIDHeight]

    group_radius: Optional[float]
    """Farthest horizontal distance from reported group location at which an aircraft in the group may be located (meters).  This value contains the "Operating Area Radius" data from the common data dictionary when group operation area is specified by point-radius."""

    group_ceiling: Optional[float]
    """Maximum altitude (meters WGS84-HAE) of Group Operation.  This value contains the "Operating Area Ceiling" data from the common data dictionary when group operation area is specified by point-radius."""

    group_floor: Optional[float]
    """Minimum altitude (meters WGS84-HAE) of Group Operation.  If not specified, ground level shall be assumed.  This value contains the "Operating Area Floor" data from the common data dictionary when group operation area is specified by point-radius."""

    group_count: Optional[int]
    """When operating a group (or formation or swarm), number of aircraft in group.  This value contains the "Operating Area Count" data from the common data dictionary when group operation area is specified by point-radius."""

    group_time_start: Optional[StringBasedDateTime]
    """Time at which a group operation starts.  This value contains the "Operation Area Start" data from the common data dictionary when group operation area is specified by point-radius."""

    group_time_end: Optional[StringBasedDateTime]
    """Time at which a group operation starts.  This value contains the "Operation Area End" data from the common data dictionary when group operation area is specified by point-radius."""


class RIDFlight(ImplicitDict):
    """Description of a remote ID flight."""

    id: RIDFlightID

    aircraft_type: RIDAircraftType
    """Generic type of aircraft."""

    current_state: Optional[RIDAircraftState]
    """The most up-to-date state of the aircraft.  Required when the aircraft is currently in the requested area unless `volumes` is specified.

    If current data is not being received from the UAS by the Service Provider, the lack of change in this field is sufficient to indicate that current data is not being received.
    """

    volumes: Optional[List[Volume4D]]
    """The set of spacetime volumes the aircraft is within.  Required if `current_state` is not specified.  The fields `time_start` and `time_end` are required if `current_state` is not specified."""

    simulated: Optional[bool]
    """If specified as true, this flight is not a physical aircraft; it is just a simulation to test the system."""

    recent_positions: Optional[List[RIDRecentAircraftPosition]]
    """A short collection of recent aircraft movement, specified only when `include_recent_positions` is true.  If `volumes` is not specified and `include_recent_positions` is true, then this field is required.

    Recent positions provided in this field must conform to requirements in the standard which generally prohibit including positions outside the requested area except transitionally when the aircraft enters or exits the requested area, and which prohibit including positions that not sufficiently recent.

    Note that a UI should not draw a connective line between two consecutive position reports that both lie outside the requested area.
    """


class PutIdentificationServiceAreaResponse(ImplicitDict):
    """Response to a request to create or update a reference to an Identification Service Area in the DSS."""

    subscribers: List[SubscriberToNotify]
    """DSS subscribers that this client now has the obligation to notify of the Identification Service Area changes just made.  This client must call POST for each provided URL according to the `/uss/identification_service_areas/{id}` path API."""

    service_area: IdentificationServiceArea
    """Resulting service area stored in DSS."""


class SearchIdentificationServiceAreasResponse(ImplicitDict):
    """Response to DSS query for Identification Service Areas in an area of interest."""

    service_areas: List[IdentificationServiceArea]
    """Identification Service Areas in the area of interest."""


class PutIdentificationServiceAreaNotificationParameters(ImplicitDict):
    """Parameters of a message informing of new full information for an Identification Service Area.  Pushed (by a client, not the DSS) directly to clients with subscriptions when another client makes a change to airspace within a cell with a subscription."""

    service_area: Optional[IdentificationServiceArea]
    """Identification Service Area that the notifying client changed or created.

    If this field is populated, the Identification Service Area was created or updated.  If this field is not populated, the Identification Service Area was deleted.
    """

    subscriptions: List[SubscriptionState]
    """Subscription(s) prompting this notification."""

    extents: Optional[Volume4D]
    """The new or updated extents of the Identification Service Area.

    Omitted if Identification Service Area was deleted.
    """


class DeleteIdentificationServiceAreaResponse(ImplicitDict):
    """Response for a request to delete an Identification Service Area."""

    service_area: IdentificationServiceArea
    """Indentification Service Area that was just deleted."""

    subscribers: List[SubscriberToNotify]
    """DSS subscribers that this client now has the obligation to notify of the Identification Service Area just deleted.  This client must call POST for each provided URL according to the `/uss/identification_service_areas` path API."""


class PutSubscriptionResponse(ImplicitDict):
    """Response for a request to create or update a subscription."""

    service_areas: Optional[List[IdentificationServiceArea]]
    """Identification Service Areas in or near the subscription area at the time of creation/update, if `identification_service_area_url` callback was specified."""

    subscription: Subscription
    """Result of the operation on the subscription."""


class GetIdentificationServiceAreaResponse(ImplicitDict):
    """Response to DSS request for the identification service area with the given id."""

    service_area: IdentificationServiceArea


class GetFlightsResponse(ImplicitDict):
    """Response to remote ID provider query for flight information in an area of interest."""

    timestamp: StringBasedDateTime
    """The remote ID service provider's timestamp for when this information was current.  RFC 3339 format, per OpenAPI specification."""

    flights: List[RIDFlight]
    """A list of all flights that have been within the requested area within the remote ID retention period.  This includes flights that are not currently within the requested area, but were within the requested area within the remote ID retention period."""


class OperationID(str, Enum):
    SearchIdentificationServiceAreas = "searchIdentificationServiceAreas"
    GetIdentificationServiceArea = "getIdentificationServiceArea"
    CreateIdentificationServiceArea = "createIdentificationServiceArea"
    UpdateIdentificationServiceArea = "updateIdentificationServiceArea"
    DeleteIdentificationServiceArea = "deleteIdentificationServiceArea"
    SearchFlights = "searchFlights"
    GetFlightDetails = "getFlightDetails"
    SearchSubscriptions = "searchSubscriptions"
    GetSubscription = "getSubscription"
    CreateSubscription = "createSubscription"
    UpdateSubscription = "updateSubscription"
    DeleteSubscription = "deleteSubscription"
    PostIdentificationServiceArea = "postIdentificationServiceArea"


OPERATIONS: Dict[OperationID, Operation] = {
    OperationID.SearchIdentificationServiceAreas: Operation(
        id="searchIdentificationServiceAreas",
        path="/v1/dss/identification_service_areas",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: SearchIdentificationServiceAreasResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            413: ErrorResponse,
        }
    ),
    OperationID.GetIdentificationServiceArea: Operation(
        id="getIdentificationServiceArea",
        path="/v1/dss/identification_service_areas/{id}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetIdentificationServiceAreaResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
        }
    ),
    OperationID.CreateIdentificationServiceArea: Operation(
        id="createIdentificationServiceArea",
        path="/v1/dss/identification_service_areas/{id}",
        verb="PUT",
        request_body_type=CreateIdentificationServiceAreaParameters,
        response_body_type={
            200: PutIdentificationServiceAreaResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            413: ErrorResponse,
        }
    ),
    OperationID.UpdateIdentificationServiceArea: Operation(
        id="updateIdentificationServiceArea",
        path="/v1/dss/identification_service_areas/{id}/{version}",
        verb="PUT",
        request_body_type=UpdateIdentificationServiceAreaParameters,
        response_body_type={
            200: PutIdentificationServiceAreaResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            413: ErrorResponse,
        }
    ),
    OperationID.DeleteIdentificationServiceArea: Operation(
        id="deleteIdentificationServiceArea",
        path="/v1/dss/identification_service_areas/{id}/{version}",
        verb="DELETE",
        request_body_type=None,
        response_body_type={
            200: DeleteIdentificationServiceAreaResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            409: ErrorResponse,
        }
    ),
    OperationID.SearchFlights: Operation(
        id="searchFlights",
        path="/v1/uss/flights",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetFlightsResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            413: ErrorResponse,
        }
    ),
    OperationID.GetFlightDetails: Operation(
        id="getFlightDetails",
        path="/v1/uss/flights/{id}/details",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetFlightDetailsResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
        }
    ),
    OperationID.SearchSubscriptions: Operation(
        id="searchSubscriptions",
        path="/v1/dss/subscriptions",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: SearchSubscriptionsResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            413: ErrorResponse,
        }
    ),
    OperationID.GetSubscription: Operation(
        id="getSubscription",
        path="/v1/dss/subscriptions/{id}",
        verb="GET",
        request_body_type=None,
        response_body_type={
            200: GetSubscriptionResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
        }
    ),
    OperationID.CreateSubscription: Operation(
        id="createSubscription",
        path="/v1/dss/subscriptions/{id}",
        verb="PUT",
        request_body_type=CreateSubscriptionParameters,
        response_body_type={
            200: PutSubscriptionResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.UpdateSubscription: Operation(
        id="updateSubscription",
        path="/v1/dss/subscriptions/{id}/{version}",
        verb="PUT",
        request_body_type=UpdateSubscriptionParameters,
        response_body_type={
            200: PutSubscriptionResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
            429: ErrorResponse,
        }
    ),
    OperationID.DeleteSubscription: Operation(
        id="deleteSubscription",
        path="/v1/dss/subscriptions/{id}/{version}",
        verb="DELETE",
        request_body_type=None,
        response_body_type={
            200: DeleteSubscriptionResponse,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            404: ErrorResponse,
            409: ErrorResponse,
        }
    ),
    OperationID.PostIdentificationServiceArea: Operation(
        id="postIdentificationServiceArea",
        path="/v1/uss/identification_service_areas/{id}",
        verb="POST",
        request_body_type=PutIdentificationServiceAreaNotificationParameters,
        response_body_type={
            204: None,
            400: ErrorResponse,
            401: ErrorResponse,
            403: ErrorResponse,
            409: ErrorResponse,
        }
    ),
}



from datetime import time
from enum import Enum
from typing import List, Any, Optional, Dict, Union
import arrow

from implicitdict import ImplicitDict, StringBasedDateTime


class Restriction(str, Enum):
    PROHIBITED = "PROHIBITED"
    REQ_AUTHORISATION = "REQ_AUTHORISATION"
    CONDITIONAL = "CONDITIONAL"
    NO_RESTRICTION = "NO_RESTRICTION"


class Reason(str, Enum):
    AIR_TRAFFIC = "AIR_TRAFFIC"
    SENSITIVE = "SENSITIVE"
    PRIVACY = "PRIVACY"
    POPULATION = "POPULATION"
    NATURE = "NATURE"
    NOISE = "NOISE"
    FOREIGN_TERRITORY = "FOREIGN_TERRITORY"
    EMERGENCY = "EMERGENCY"
    OTHER = "OTHER"


class YESNO(str, Enum):
    YES = "YES"
    NO = "NO"


class Purpose(str, Enum):
    AUTHORIZATION = "AUTHORIZATION"
    NOTIFICATION = "NOTIFICATION"
    INFORMATION = "INFORMATION"


class UASZoneAuthority(ImplicitDict):
    name: Optional[str]  # max length: 200
    service: Optional[str]  # max length: 200
    email: Optional[str]
    contactName: Optional[str]  # max length: 200
    siteURL: Optional[str]
    phone: Optional[str]  # max length: 200
    purpose: Optional[Purpose]
    intervalBefore: Optional[str]


class VerticalReferenceType(str, Enum):
    AGL = "AGL"
    AMSL = "AMSL"


class HorizontalProjectionType(str, Enum):
    Circle = "Circle"
    Polygon = "Polygon"


class CircleOrPolygonType(ImplicitDict):
    type: HorizontalProjectionType
    center: Optional[List[float]]  # 2 items. Coordinates: lat, lng
    radius: Optional[float]  # > 0
    coordinates: Optional[List[List[List[float]]]]  # List of polygons -> List of points: min 4 items -> Coordinates: lat, lng



class UomDimensions(str, Enum):
    M = "M"
    FT = "FT"


class UASZoneAirspaceVolume(ImplicitDict):
    uomDimensions: UomDimensions
    lowerLimit: Optional[int]
    lowerVerticalReference: VerticalReferenceType
    upperLimit: Optional[int]
    upperVerticalReference: VerticalReferenceType
    horizontalProjection: CircleOrPolygonType


class WeekDateType(str, Enum):
    MON = "MON"
    TUE = "TUE"
    WED = "WED"
    THU = "THU"
    FRI = "FRI"
    SAT = "SAT"
    SUN = "SUN"
    ANY = "ANY"


class ED269TimeType(str):
    """String that allows values which describe a time in ED-269 flavour of ISO 8601 format.

    ED-269 standard specifies that a time instant type should be in the form of hh:mmS where S is
    the timezone. However, examples are using the following format: 00:00:00.00Z
    This class supports both formats as inputs and uses the long form as the output format.
    """

    time: time
    """`time` representation of the str value with timezone"""

    def __new__(cls, value: Union[str, time]):
        if isinstance(value, str):
            t = arrow.get(value, ["HH:mm:ss.SZ", "HH:mmZ"]).timetz()
        else:
            t = value
        str_value = str.__new__(
            cls, t.strftime("%H:%M:%S.%f")[:11] + t.strftime("%z").replace("+0000", "Z")
        )
        str_value.time = t
        return str_value


class DailyPeriod(ImplicitDict):
    day: List[WeekDateType]  # min items: 1, max items: 7
    startTime: ED269TimeType
    endTime: ED269TimeType


class ApplicableTimePeriod(ImplicitDict):
    permanent: YESNO
    startDateTime: Optional[StringBasedDateTime]
    endDateTime: Optional[StringBasedDateTime]
    schedule: Optional[List[DailyPeriod]]  # min items: 1


class UASZoneVersion(ImplicitDict):
    title: Optional[str]
    identifier: str  # max length: 7
    country: str  # length: 3
    name: Optional[str]  # max length: 200
    type: str
    restriction: Restriction
    restrictionConditions: Optional[List[str]]
    region: Optional[int]
    reason: Optional[List[Reason]]  # max length: 9
    otherReasonInfo: Optional[str]  # max length: 30
    regulationExemption: Optional[YESNO]
    uSpaceClass: Optional[str]  # max length: 100
    message: Optional[str]  # max length: 200
    applicability: List[ApplicableTimePeriod]
    zoneAuthority: List[UASZoneAuthority]
    geometry: List[UASZoneAirspaceVolume]  # min items: 1
    extendedProperties: Optional[Any]


class ED269Schema(ImplicitDict):
    title: Optional[str]
    description: Optional[str]
    features: List[UASZoneVersion]

    @staticmethod
    def from_dict(raw_data: Dict) -> "ED269Schema":
        return ImplicitDict.parse(raw_data, ED269Schema)



from __future__ import annotations
import random
import string
from typing import Optional


class OperatorRegistrationNumber(str):
    """Represents an operator registration number as formatted according to the EN4709-02 standard."""

    registration_number_code_points = "0123456789abcdefghijklmnopqrstuvwxyz"
    prefix_length = 3
    base_id_length = 12
    final_random_string_length = 3
    checksum_length = 1
    public_number_length = prefix_length + base_id_length + checksum_length
    dash_length = 1
    full_number_length = public_number_length + dash_length + final_random_string_length

    @property
    def checksum_control(self) -> str:
        return self.split("-")[0]

    @property
    def prefix(self) -> str:
        return self[0 : OperatorRegistrationNumber.prefix_length]

    @property
    def base_id(self) -> str:
        return self[
               OperatorRegistrationNumber.prefix_length : OperatorRegistrationNumber.prefix_length
                                                          + OperatorRegistrationNumber.base_id_length
               ]

    @property
    def checksum(self) -> str:
        return self[
               OperatorRegistrationNumber.prefix_length
               + OperatorRegistrationNumber.base_id_length :
               ][0]

    @property
    def final_random_string(self):
        return self[-OperatorRegistrationNumber.final_random_string_length :]

    @property
    def valid(self) -> bool:
        # PPPBBBBBBBBBBBBC-FFF
        # P = prefix, B = base ID, C = checksum, F = final random string
        if len(self) != OperatorRegistrationNumber.full_number_length:
            return False
        if self[OperatorRegistrationNumber.public_number_length] != "-":
            return False
        if not all(
                c in OperatorRegistrationNumber.registration_number_code_points
                for c in self.base_id
        ):
            return False
        if not all(
                c in OperatorRegistrationNumber.registration_number_code_points
                for c in self.final_random_string
        ):
            return False
        checksum = OperatorRegistrationNumber.generate_checksum(
            self.base_id, self.final_random_string
        )
        return self.checksum == checksum

    def make_invalid_by_changing_final_control_string(
            self, r: Optional[random.Random] = None
    ) -> OperatorRegistrationNumber:
        """A method to generate an invalid Operator Registration number by replacing the control string"""
        if r is None:
            r = random
        while True:
            new_random_string = "".join(
                r.choice(string.ascii_lowercase)
                for _ in range(OperatorRegistrationNumber.final_random_string_length)
            )
            result = OperatorRegistrationNumber(
                self.checksum_control + "-" + new_random_string
            )
            if not result.valid:
                return result

    @staticmethod
    def validate_prefix(prefix: str) -> None:
        if len(prefix) != OperatorRegistrationNumber.prefix_length:
            raise ValueError(
                "Prefix of an operator registration number must be {} characters long rather than {}".format(
                    OperatorRegistrationNumber.prefix_length, len(prefix)
                )
            )

    @staticmethod
    def validate_base_id(base_id: str) -> None:
        if len(base_id) != OperatorRegistrationNumber.base_id_length:
            raise ValueError(
                "Base ID of an operator registration number must be {} characters long rather than {}".format(
                    OperatorRegistrationNumber.base_id_length, len(base_id)
                )
            )
        if not all(
                c in OperatorRegistrationNumber.registration_number_code_points
                for c in base_id
        ):
            raise ValueError(
                "Base ID of an operator registration number must be alphanumeric"
            )

    @staticmethod
    def validate_final_random_string(final_random_string: str) -> None:
        if (
                len(final_random_string)
                != OperatorRegistrationNumber.final_random_string_length
        ):
            raise ValueError(
                "Final random string of an operator registration number must be {} characters long rather than {}".format(
                    OperatorRegistrationNumber.final_random_string_length,
                    len(final_random_string),
                )
            )
        if not all(
                c in OperatorRegistrationNumber.registration_number_code_points
                for c in final_random_string
        ):
            raise ValueError(
                "Final random string of an operator registration number must be alphanumeric"
            )

    @staticmethod
    def generate_checksum(base_id: str, final_random_string: str) -> str:
        OperatorRegistrationNumber.validate_base_id(base_id)
        OperatorRegistrationNumber.validate_final_random_string(final_random_string)
        raw_id = base_id + final_random_string

        full_sum = 0
        multiplier = 2
        n = len(OperatorRegistrationNumber.registration_number_code_points)
        for c in raw_id:
            v = OperatorRegistrationNumber.registration_number_code_points.index(c)
            quotient, remainder = divmod(v * multiplier, n)
            full_sum += quotient + remainder
            multiplier = 3 - multiplier

        control_number = -full_sum % n
        return OperatorRegistrationNumber.registration_number_code_points[
            control_number
        ]

    @staticmethod
    def generate_valid(prefix: str, r: Optional[random.Random] = None) -> OperatorRegistrationNumber:
        """Generate a random operator registration number with the specified prefix"""
        if r is None:
            r = random
        final_random_string = "".join(
            r.choice(string.ascii_lowercase)
            for _ in range(OperatorRegistrationNumber.final_random_string_length)
        )
        base_id = "".join(
            r.choice(string.ascii_lowercase + string.digits)
            for _ in range(OperatorRegistrationNumber.base_id_length)
        )
        return OperatorRegistrationNumber.from_components(
            prefix, base_id, final_random_string
        )

    @staticmethod
    def from_components(
            prefix: str, base_id: str, final_random_string: str
    ) -> OperatorRegistrationNumber:
        """Constructs a standard operator registration number from the provided components"""
        OperatorRegistrationNumber.validate_prefix(prefix)
        OperatorRegistrationNumber.validate_base_id(base_id)
        if (
                len(final_random_string)
                != OperatorRegistrationNumber.final_random_string_length
        ):
            raise ValueError(
                "Prefix of an operator registration number must be {} characters long rather than {}".format(
                    OperatorRegistrationNumber.final_random_string_length,
                    len(final_random_string),
                )
            )
        checksum = OperatorRegistrationNumber.generate_checksum(
            base_id, final_random_string
        )
        return OperatorRegistrationNumber(
            prefix + base_id + checksum + "-" + final_random_string
        )



from dataclasses import dataclass
from typing import List, Dict, Optional


@dataclass
class Operation(object):
    id: str
    path: str
    verb: str
    request_body_type: Optional[str]
    response_body_type: Dict[int, Optional[str]]

    @property
    def name(self) -> str:
        return self.id[0].upper() + self.id[1:]


def get_operations(spec: dict) -> List[Operation]:
    ref_prefix = '#/components/schemas/'
    operations: List[Operation] = []
    all_verbs = {"get", "put", "post", "delete", "patch"}
    for path, body in spec["paths"].items():
        for k, v in body.items():
            if k not in all_verbs:
                continue

            ref = v.get("requestBody", {}).get("content", {}).get("application/json", {}).get("schema", {}).get("$ref", '')
            request_body_type = ref[len(ref_prefix):] if ref.startswith(ref_prefix) else None

            response_body_type = {}
            for code, response_content in v["responses"].items():
                ref = response_content.get("content", {}).get("application/json", {}).get("schema", {}).get("$ref", '')
                response_body_type[int(code)] = ref[len(ref_prefix):] if ref.startswith(ref_prefix) else None

            op_id = v.get("operationId", None)
            if not op_id:
                print(f'WARNING: {k} {path} is missing an operationId')
                path_parts = path.split('/')
                if path_parts[-1].startswith('{') or '_' in path_parts[-1]:
                    raise NotImplementedError('Cannot construct operationId')
                op_id = k.lower() + path_parts[-1][0].upper() + path_parts[-1][1:]
            operations.append(Operation(
                id=op_id,
                path=path,
                verb=k.upper(),
                request_body_type=request_body_type,
                response_body_type=response_body_type,
            ))
    return operations



import json
import random
import pytest

from uas_standards.en4709_02 import OperatorRegistrationNumber


def test_basic_usage():
    r = random.Random(12345)

    rn = OperatorRegistrationNumber.generate_valid('EXM', r)
    assert rn.valid
    OperatorRegistrationNumber.validate_prefix(rn.prefix)
    OperatorRegistrationNumber.validate_base_id(rn.base_id)
    OperatorRegistrationNumber.validate_final_random_string(rn.final_random_string)

    rn2 = OperatorRegistrationNumber.from_components(rn.prefix, rn.base_id, rn.final_random_string)
    assert rn2.valid
    assert rn2 == rn

    plain_str = json.loads(json.dumps({'rn': rn}))['rn']
    rn3 = OperatorRegistrationNumber(plain_str)
    assert rn3.valid
    assert rn3 == rn

    rn_invalid = rn.make_invalid_by_changing_final_control_string(r)
    assert rn.valid
    assert not rn_invalid.valid
    OperatorRegistrationNumber.validate_prefix(rn_invalid.prefix)
    OperatorRegistrationNumber.validate_base_id(rn_invalid.base_id)
    OperatorRegistrationNumber.validate_final_random_string(rn_invalid.final_random_string)

    with pytest.raises(ValueError):
        OperatorRegistrationNumber.validate_prefix('US')

    OperatorRegistrationNumber.validate_base_id('aaaaaaaaaaaa')
    with pytest.raises(ValueError):
        OperatorRegistrationNumber.validate_base_id('aaaaaaaaaaa')
    with pytest.raises(ValueError):
        OperatorRegistrationNumber.validate_base_id('aaaaaaaaaaaA')

    OperatorRegistrationNumber.validate_final_random_string('aaa')
    with pytest.raises(ValueError):
        OperatorRegistrationNumber.validate_final_random_string('aa')
    with pytest.raises(ValueError):
        OperatorRegistrationNumber.validate_final_random_string('aaA')
